@@ -80,6 +80,7 @@ enum trx_mod_type {
 
 /* A set of measurements belonging to one Uplink burst */
 struct l1sched_meas_set {
+	uint32_t		fn;		/* TDMA frame number */
 	int16_t			toa256;		/* Timing of Arrival (1/256 of a symbol) */
 	int16_t			ci_cb;		/* Carrier-to-Interference (cB) */
 	float			rssi;		/* RSSI (dBm) */
@@ -102,7 +103,6 @@ struct l1sched_chan_state {
 	uint8_t			ul_mask;	/* mask of received bursts */
 
 	/* loss detection */
-	uint8_t			lost_frames;	/* how many L2 frames were lost */
 	uint32_t		last_tdma_fn;	/* last processed TDMA frame number */
 	uint32_t		proc_tdma_fs;	/* how many TDMA frames were processed */
 	uint32_t		lost_tdma_fs;	/* how many TDMA frames were lost */
@@ -113,8 +113,8 @@ struct l1sched_chan_state {
 	/* AMR */
 	uint8_t			codec[4];	/* 4 possible codecs for amr */
 	int			codecs;		/* number of possible codecs */
-	float			ber_sum;	/* sum of bit error rates */
-	int			ber_num;	/* number of bit error rates */
+	int			lqual_cb_sum;	/* sum of link quality samples (in cB) */
+	int			lqual_cb_num;	/* number of link quality samples */
 	uint8_t			ul_ft;		/* current uplink FT index */
 	uint8_t			dl_ft;		/* current downlink FT index */
 	uint8_t			ul_cmr;		/* current uplink CMR index */
@@ -125,8 +125,6 @@ struct l1sched_chan_state {
 	/* TCH/H */
 	uint8_t			dl_ongoing_facch; /* FACCH/H on downlink */
 	uint8_t			ul_ongoing_facch; /* FACCH/H on uplink */
-	struct l1sched_meas_set meas_avg_facch;   /* measurement results for last FACCH */
-	uint16_t		ber10k_facch;	  /* bit error rate for last FACCH */
 
 	uint8_t			dl_facch_bursts;  /* number of remaining DL FACCH bursts */
 
@@ -233,7 +231,7 @@ struct trx_sched_multiframe {
 int find_sched_mframe_idx(enum gsm_phys_chan_config pchan, uint8_t tn);
 
 /*! Determine if given frame number contains SACCH (true) or other (false) burst */
-bool trx_sched_is_sacch_fn(struct gsm_bts_trx_ts *ts, uint32_t fn, bool uplink);
+bool trx_sched_is_sacch_fn(const struct gsm_bts_trx_ts *ts, uint32_t fn, bool uplink);
 extern const struct trx_sched_multiframe trx_sched_multiframes[];
 
 #define TRX_BI_F_NOPE_IND	(1 << 0)
@@ -306,18 +304,20 @@ int trx_sched_ul_burst(struct l1sched_ts *l1ts, struct trx_ul_burst_ind *bi);
 
 /* Averaging mode for trx_sched_meas_avg() */
 enum sched_meas_avg_mode {
-	/* last 4 bursts (default for xCCH, TCH/H, PTCCH and PDTCH) */
-	SCHED_MEAS_AVG_M_QUAD,
+	/* last 4 bursts (default for xCCH, PTCCH and PDTCH) */
+	SCHED_MEAS_AVG_M_S4N4,
 	/* last 8 bursts (default for TCH/F and FACCH/F) */
-	SCHED_MEAS_AVG_M_OCTO,
+	SCHED_MEAS_AVG_M_S8N8,
+	/* first 4 of last 6 bursts (default for TCH/H) */
+	SCHED_MEAS_AVG_M_S6N4,
 	/* last 6 bursts (default for FACCH/H) */
-	SCHED_MEAS_AVG_M_SIX,
+	SCHED_MEAS_AVG_M_S6N6,
 	/* first 4 of last 8 bursts */
-	SCHED_MEAS_AVG_M8_FIRST_QUAD,
+	SCHED_MEAS_AVG_M_S8N4,
 	/* first 2 of last 6 bursts */
-	SCHED_MEAS_AVG_M6_FIRST_TWO,
+	SCHED_MEAS_AVG_M_S6N2,
 	/* middle 2 of last 6 bursts */
-	SCHED_MEAS_AVG_M6_MIDDLE_TWO,
+	SCHED_MEAS_AVG_M_S4N2,
 };
 
 void trx_sched_meas_push(struct l1sched_chan_state *chan_state,
@@ -325,3 +325,5 @@ void trx_sched_meas_push(struct l1sched_chan_state *chan_state,
 void trx_sched_meas_avg(const struct l1sched_chan_state *chan_state,
 			struct l1sched_meas_set *avg,
 			enum sched_meas_avg_mode mode);
+uint32_t trx_sched_lookup_fn(const struct l1sched_chan_state *chan_state,
+			     const unsigned int shift);
