@@ -32,9 +32,11 @@
 #include <osmocom/core/timer.h>
 #include <osmocom/core/msgb.h>
 #include <osmocom/core/talloc.h>
+#include <osmocom/core/tdef.h>
 #include <osmocom/core/stats.h>
 #include <osmocom/core/rate_ctr.h>
 #include <osmocom/gsm/protocol/gsm_12_21.h>
+#include <osmocom/gsm/gsm48.h>
 #include <osmocom/gsm/lapdm.h>
 #include <osmocom/trau/osmo_ortp.h>
 
@@ -48,9 +50,13 @@
 #include <osmo-bts/oml.h>
 #include <osmo-bts/signal.h>
 #include <osmo-bts/dtx_dl_amr_fsm.h>
+#include <osmo-bts/cbch.h>
+#include <osmo-bts/bts_shutdown_fsm.h>
+#include <osmo-bts/nm_common_fsm.h>
+#include <osmo-bts/power_control.h>
 
-#define MIN_QUAL_RACH    5.0f   /* at least  5 dB C/I */
-#define MIN_QUAL_NORM   -0.5f   /* at least -1 dB C/I */
+#define MIN_QUAL_RACH	 50 /* minimum link quality (in centiBels) for Access Bursts */
+#define MIN_QUAL_NORM	 -5 /* minimum link quality (in centiBels) for Normal Bursts */
 
 static void bts_update_agch_max_queue_length(struct gsm_bts *bts);
 
@@ -84,6 +90,8 @@ static int bts_signal_cbfn(unsigned int subsys, unsigned int signal,
 static const struct rate_ctr_desc bts_ctr_desc[] = {
 	[BTS_CTR_PAGING_RCVD] =		{"paging:rcvd", "Received paging requests (Abis)"},
 	[BTS_CTR_PAGING_DROP] =		{"paging:drop", "Dropped paging requests (Abis)"},
+	[BTS_CTR_PAGING_DROP_PS] =	{"paging:drop-ps", "Dropped paging requests (PS/PCU)"},
+	[BTS_CTR_PAGING_CONG] =		{"paging:cong", "Paging congestion detected (Abis)"},
 	[BTS_CTR_PAGING_SENT] =		{"paging:sent", "Sent paging requests (Um)"},
 
 	[BTS_CTR_RACH_RCVD] =		{"rach:rcvd", "Received RACH requests (Um)"},
@@ -104,11 +112,202 @@ static const struct rate_ctr_group_desc bts_ctrg_desc = {
 	bts_ctr_desc
 };
 
-/* Initialize the BTS (and TRX) data structures, called before config
+static const struct rate_ctr_desc cbch_ctr_desc[] = {
+	[CBCH_CTR_RCVD_QUEUED] =	{"cbch:rcvd_queued", "Received + queued CBCH messages (Abis)" },
+	[CBCH_CTR_RCVD_DROPPED] =	{"cbch:rcvd_dropped", "Received + dropped CBCH messages (Abis)" },
+
+	[CBCH_CTR_SENT_SINGLE] =	{"cbch:sent_single", "Sent single CBCH messages (Um)" },
+	[CBCH_CTR_SENT_DEFAULT] =	{"cbch:sent_default", "Sent default CBCH messages (Um)" },
+	[CBCH_CTR_SENT_NULL] =		{"cbch:sent_null", "Sent NULL CBCH messages (Um)" },
+};
+static const struct rate_ctr_group_desc cbch_ctrg_desc = {
+	"cbch",
+	"cell broadcast channel",
+	OSMO_STATS_CLASS_GLOBAL,
+	ARRAY_SIZE(cbch_ctr_desc),
+	cbch_ctr_desc
+};
+
+struct osmo_tdef bts_T_defs[] = {
+	/* T-1: FIXME: Ideally should be dynamically calculated per trx at
+	 * shutdown start based on params below, and highest trx value taken:
+	 * + VTY's power-ramp step-interval.
+	 * + Amount of steps needed (taking into account how many dB each step moves).
+	 * + Extra time to get response back for each step.
+	 * For now we simply give 5 mins, which should be enough for any
+	 * acceptable setup, while still ensuring will timeout at some point if
+	 * something fails in the ramp down procedure.
+	 */
+	{ .T=-1, .default_val=300, .desc="Time after which osmo-bts exits if regular ramp down during shut down process does not finish (s)" },
+	{ .T=-2, .default_val=3, .desc="Time after which osmo-bts exits if requesting transceivers to stop during shut down process does not finish (s)" },
+	{}
+};
+
+struct osmo_tdef abis_T_defs[] = {
+	{ .T=-15, .default_val=0, .unit=OSMO_TDEF_MS, .desc="Time to wait between Channel Activation and dispatching a cached early Immediate Assignment" },
+	{}
+};
+
+static const uint8_t bts_nse_timer_default[] = { 3, 3, 3, 3, 30, 3, 10 };
+static const uint8_t bts_cell_timer_default[] =
+				{ 3, 3, 3, 3, 3, 10, 3, 10, 3, 10, 3 };
+static const struct gprs_rlc_cfg rlc_cfg_default = {
+	.parameter = {
+		[RLC_T3142] = 20,
+		[RLC_T3169] = 5,
+		[RLC_T3191] = 5,
+		[RLC_T3193] = 160, /* 10ms */
+		[RLC_T3195] = 5,
+		[RLC_N3101] = 10,
+		[RLC_N3103] = 4,
+		[RLC_N3105] = 8,
+		[CV_COUNTDOWN] = 15,
+		[T_DL_TBF_EXT] = 250 * 10, /* ms */
+		[T_UL_TBF_EXT] = 250 * 10, /* ms */
+	},
+	.paging = {
+		.repeat_time = 5 * 50, /* ms */
+		.repeat_count = 3,
+	},
+	.cs_mask = 0x1fff,
+	.initial_cs = 2,
+	.initial_mcs = 6,
+};
+
+const struct value_string osmo_bts_variant_names[_NUM_BTS_VARIANT + 1] = {
+	{ BTS_UNKNOWN,		"unknown" },
+	{ BTS_OSMO_LITECELL15,	"osmo-bts-lc15" },
+	{ BTS_OSMO_OC2G,	"osmo-bts-oc2g" },
+	{ BTS_OSMO_OCTPHY,	"osmo-bts-octphy" },
+	{ BTS_OSMO_SYSMO,	"osmo-bts-sysmo" },
+	{ BTS_OSMO_TRX,		"osmo-bts-trx" },
+	{ BTS_OSMO_VIRTUAL,	"osmo-bts-virtual" },
+	{ BTS_OSMO_OMLDUMMY,	"osmo-bts-omldummy" },
+	{ 0, NULL }
+};
+
+const char *btsvariant2str(enum gsm_bts_type_variant v)
+{
+	return get_value_string(osmo_bts_variant_names, v);
+}
+
+const struct value_string bts_attribute_names[] = {
+	OSMO_VALUE_STRING(BTS_TYPE_VARIANT),
+	OSMO_VALUE_STRING(BTS_SUB_MODEL),
+	OSMO_VALUE_STRING(TRX_PHY_VERSION),
+	{ 0, NULL }
+};
+
+const char *btsatttr2str(enum bts_attribute v)
+{
+	return get_value_string(bts_attribute_names, v);
+}
+
+const struct value_string bts_impl_flag_desc[] = {
+	{ BTS_INTERNAL_FLAG_MS_PWR_CTRL_DSP,	"DSP/HW based MS Power Control Loop" },
+	{ BTS_INTERNAL_FLAG_MEAS_PAYLOAD_COMB,	"Measurement and Payload data combined" },
+	{ 0, NULL }
+};
+
+static int gsm_bts_talloc_destructor(struct gsm_bts *bts)
+{
+	if (bts->site_mgr.mo.fi) {
+		osmo_fsm_inst_free(bts->site_mgr.mo.fi);
+		bts->site_mgr.mo.fi = NULL;
+	}
+	if (bts->mo.fi) {
+		osmo_fsm_inst_free(bts->mo.fi);
+		bts->mo.fi = NULL;
+	}
+	if (bts->shutdown_fi) {
+		osmo_fsm_inst_free(bts->shutdown_fi);
+		bts->shutdown_fi = NULL;
+	}
+	return 0;
+}
+
+struct gsm_bts *gsm_bts_alloc(void *ctx, uint8_t bts_num)
+{
+	struct gsm_bts *bts = talloc_zero(ctx, struct gsm_bts);
+	int i;
+
+	if (!bts)
+		return NULL;
+
+	talloc_set_destructor(bts, gsm_bts_talloc_destructor);
+
+	bts->nr = bts_num;
+	bts->num_trx = 0;
+	INIT_LLIST_HEAD(&bts->trx_list);
+	bts->ms_max_power = 15;	/* dBm */
+
+	bts->T_defs = bts_T_defs;
+	osmo_tdefs_reset(bts->T_defs);
+	osmo_tdefs_reset(abis_T_defs);
+	bts->shutdown_fi = osmo_fsm_inst_alloc(&bts_shutdown_fsm, bts, bts,
+					       LOGL_INFO, NULL);
+	osmo_fsm_inst_update_id_f(bts->shutdown_fi, "bts%d", bts->nr);
+
+	bts->site_mgr.mo.fi = osmo_fsm_inst_alloc(&nm_bts_sm_fsm, bts, &bts->site_mgr,
+						  LOGL_INFO, "bts_sm");
+	gsm_mo_init(&bts->site_mgr.mo, bts, NM_OC_SITE_MANAGER,
+		    0xff, 0xff, 0xff);
+
+	bts->mo.fi = osmo_fsm_inst_alloc(&nm_bts_fsm, bts, bts,
+					 LOGL_INFO, NULL);
+	osmo_fsm_inst_update_id_f(bts->mo.fi, "bts%d", bts->nr);
+	gsm_mo_init(&bts->mo, bts, NM_OC_BTS, bts->nr, 0xff, 0xff);
+
+	for (i = 0; i < ARRAY_SIZE(bts->gprs.nsvc); i++) {
+		bts->gprs.nsvc[i].bts = bts;
+		bts->gprs.nsvc[i].id = i;
+		gsm_mo_init(&bts->gprs.nsvc[i].mo, bts, NM_OC_GPRS_NSVC,
+				bts->nr, i, 0xff);
+	}
+	memcpy(&bts->gprs.nse.timer, bts_nse_timer_default,
+		sizeof(bts->gprs.nse.timer));
+	gsm_mo_init(&bts->gprs.nse.mo, bts, NM_OC_GPRS_NSE,
+			bts->nr, 0xff, 0xff);
+	memcpy(&bts->gprs.cell.timer, bts_cell_timer_default,
+		sizeof(bts->gprs.cell.timer));
+	gsm_mo_init(&bts->gprs.cell.mo, bts, NM_OC_GPRS_CELL,
+			bts->nr, 0xff, 0xff);
+	memcpy(&bts->gprs.cell.rlc_cfg, &rlc_cfg_default,
+		sizeof(bts->gprs.cell.rlc_cfg));
+
+	/* create our primary TRX. It will be initialized during bts_init() */
+	bts->c0 = gsm_bts_trx_alloc(bts);
+	if (!bts->c0) {
+		talloc_free(bts);
+		return NULL;
+	}
+	bts->c0->ts[0].pchan = GSM_PCHAN_CCCH_SDCCH4;
+
+	bts->features = bitvec_alloc(MAX_BTS_FEATURES / 8, bts);
+	OSMO_ASSERT(bts->features != NULL);
+
+	return bts;
+}
+
+struct gsm_bts *gsm_bts_num(const struct gsm_network *net, int num)
+{
+	struct gsm_bts *bts;
+
+	if (num >= net->num_bts)
+		return NULL;
+
+	llist_for_each_entry(bts, &net->bts_list, list) {
+		if (bts->nr == num)
+			return bts;
+	}
+
+	return NULL;
+}
+
+/* Initialize the BTS data structures, called before config
  * file reading */
 int bts_init(struct gsm_bts *bts)
 {
-	struct gsm_bts_trx *trx;
 	int rc, i;
 	static int initialized = 0;
 	void *tall_rtp_ctx;
@@ -122,6 +321,10 @@ int bts_init(struct gsm_bts *bts)
 	bts->agch_queue.length = 0;
 
 	bts->ctrs = rate_ctr_group_alloc(bts, &bts_ctrg_desc, bts->nr);
+	if (!bts->ctrs) {
+		llist_del(&bts->list);
+		return -1;
+	}
 
 	/* enable management with default levels,
 	 * raise threshold to GSM_BTS_AGCH_QUEUE_THRESH_LEVEL_DISABLE to
@@ -133,12 +336,19 @@ int bts_init(struct gsm_bts *bts)
 
 	/* configurable via VTY */
 	bts->paging_state = paging_init(bts, 200, 0);
-	bts->ul_power_target = -75;	/* dBm default */
 	bts->rtp_jitter_adaptive = false;
+	bts->rtp_port_range_start = 16384;
+	bts->rtp_port_range_end = 17407;
+	bts->rtp_port_range_next = bts->rtp_port_range_start;
+	bts->rtp_ip_dscp = -1;
+	bts->rtp_priority = -1;
+
+	/* Default (fall-back) MS/BS Power control parameters */
+	power_ctrl_params_def_reset(&bts->bs_dpc_params, true);
+	power_ctrl_params_def_reset(&bts->ms_dpc_params, false);
 
 	/* configurable via OML */
 	bts->load.ccch.load_ind_period = 112;
-	load_timer_start(bts);
 	bts->rtp_jitter_buf_ms = 100;
 	bts->max_ta = 63;
 	bts->ny1 = 4;
@@ -151,49 +361,41 @@ int bts_init(struct gsm_bts *bts)
 		bts->t200_ms[i] = oml_default_t200_ms[i];
 
 	/* default RADIO_LINK_TIMEOUT */
-	bts->radio_link_timeout = 32;
+	bts->radio_link_timeout.oml = 32;
+	bts->radio_link_timeout.current = bts->radio_link_timeout.oml;
 
 	/* Start with the site manager */
-	oml_mo_state_init(&bts->site_mgr.mo, NM_OPSTATE_ENABLED, NM_AVSTATE_OK);
+	oml_mo_state_init(&bts->site_mgr.mo, NM_OPSTATE_DISABLED, NM_AVSTATE_NOT_INSTALLED);
+	oml_mo_state_init(&bts->mo, NM_OPSTATE_DISABLED, NM_AVSTATE_NOT_INSTALLED);
 
-	/* set BTS to dependency */
-	oml_mo_state_init(&bts->mo, -1, NM_AVSTATE_DEPENDENCY);
-	oml_mo_state_init(&bts->gprs.nse.mo, -1, NM_AVSTATE_DEPENDENCY);
-	oml_mo_state_init(&bts->gprs.cell.mo, -1, NM_AVSTATE_DEPENDENCY);
-	oml_mo_state_init(&bts->gprs.nsvc[0].mo, -1, NM_AVSTATE_DEPENDENCY);
+	/* set BTS attr to dependency */
+	oml_mo_state_init(&bts->gprs.nse.mo, NM_OPSTATE_DISABLED, NM_AVSTATE_DEPENDENCY);
+	oml_mo_state_init(&bts->gprs.cell.mo, NM_OPSTATE_DISABLED, NM_AVSTATE_DEPENDENCY);
+	oml_mo_state_init(&bts->gprs.nsvc[0].mo, NM_OPSTATE_DISABLED, NM_AVSTATE_DEPENDENCY);
 	oml_mo_state_init(&bts->gprs.nsvc[1].mo, NM_OPSTATE_DISABLED, NM_AVSTATE_OFF_LINE);
 
-	/* initialize bts data structure */
-	llist_for_each_entry(trx, &bts->trx_list, list) {
-		struct trx_power_params *tpp = &trx->power_params;
-		int i;
-
-		for (i = 0; i < ARRAY_SIZE(trx->ts); i++) {
-			struct gsm_bts_trx_ts *ts = &trx->ts[i];
-			int k;
-
-			for (k = 0; k < ARRAY_SIZE(ts->lchan); k++) {
-				struct gsm_lchan *lchan = &ts->lchan[k];
-				INIT_LLIST_HEAD(&lchan->dl_tch_queue);
-			}
-		}
-		/* Default values for the power adjustments */
-		tpp->ramp.max_initial_pout_mdBm = to_mdB(0);
-		tpp->ramp.step_size_mdB = to_mdB(2);
-		tpp->ramp.step_interval_sec = 1;
-	}
-
 	/* allocate a talloc pool for ORTP to ensure it doesn't have to go back
 	 * to the libc malloc all the time */
 	tall_rtp_ctx = talloc_pool(tall_bts_ctx, 262144);
 	osmo_rtp_init(tall_rtp_ctx);
 
+	/* features implemented in 'common', available for all models,
+	 * order alphabetically */
+	osmo_bts_set_feature(bts->features, BTS_FEAT_ABIS_OSMO_PCU);
+	osmo_bts_set_feature(bts->features, BTS_FEAT_CCN);
+	osmo_bts_set_feature(bts->features, BTS_FEAT_DYN_TS_SDCCH8);
+	osmo_bts_set_feature(bts->features, BTS_FEAT_ETWS_PN);
+	osmo_bts_set_feature(bts->features, BTS_FEAT_IPV6_NSVC);
+	osmo_bts_set_feature(bts->features, BTS_FEAT_PAGING_COORDINATION);
+
 	rc = bts_model_init(bts);
 	if (rc < 0) {
 		llist_del(&bts->list);
 		return rc;
 	}
 
+	/* TRX0 was allocated early during gsm_bts_alloc, not later through VTY */
+	bts_model_trx_init(bts->c0);
 	bts_gsmnet.num_bts++;
 
 	if (!initialized) {
@@ -201,59 +403,40 @@ int bts_init(struct gsm_bts *bts)
 		initialized = 1;
 	}
 
-	INIT_LLIST_HEAD(&bts->smscb_state.queue);
-	INIT_LLIST_HEAD(&bts->oml_queue);
+	INIT_LLIST_HEAD(&bts->smscb_basic.queue);
+	bts->smscb_basic.ctrs = rate_ctr_group_alloc(bts, &cbch_ctrg_desc, 0);
+	OSMO_ASSERT(bts->smscb_basic.ctrs);
+	INIT_LLIST_HEAD(&bts->smscb_extended.queue);
+	bts->smscb_extended.ctrs = rate_ctr_group_alloc(bts, &cbch_ctrg_desc, 1);
+	OSMO_ASSERT(bts->smscb_extended.ctrs);
+	bts->smscb_queue_max_len = 15;
+	bts->smscb_queue_tgt_len = 2;
+	bts->smscb_queue_hyst = 2;
+
+	INIT_LLIST_HEAD(&bts->bsc_oml_hosts);
 
 	/* register DTX DL FSM */
 	rc = osmo_fsm_register(&dtx_dl_amr_fsm);
 	OSMO_ASSERT(rc == 0);
 
+	bts->fn_stats.min = INT32_MAX;
+	bts->fn_stats.max = INT32_MIN;
+	bts->fn_stats.avg_count = 0;
+	bts->fn_stats.avg_window = 256;
+
 	return rc;
 }
 
-static void shutdown_timer_cb(void *data)
-{
-	fprintf(stderr, "Shutdown timer expired\n");
-	exit(42);
-}
-
-static struct osmo_timer_list shutdown_timer = {
-	.cb = &shutdown_timer_cb,
-};
-
-void bts_shutdown(struct gsm_bts *bts, const char *reason)
-{
-	struct gsm_bts_trx *trx;
-
-	if (osmo_timer_pending(&shutdown_timer)) {
-		LOGP(DOML, LOGL_NOTICE,
-			"BTS is already being shutdown.\n");
-		return;
-	}
-
-	LOGP(DOML, LOGL_NOTICE, "Shutting down BTS %u, Reason %s\n",
-		bts->nr, reason);
-
-	llist_for_each_entry_reverse(trx, &bts->trx_list, list) {
-		bts_model_trx_deact_rf(trx);
-		bts_model_trx_close(trx);
-	}
-
-	/* shedule a timer to make sure select loop logic can run again
-	 * to dispatch any pending primitives */
-	osmo_timer_schedule(&shutdown_timer, 3, 0);
-}
-
 /* main link is established, send status report */
 int bts_link_estab(struct gsm_bts *bts)
 {
 	int i, j;
 
-	LOGP(DSUM, LOGL_INFO, "Main link established, sending Status'.\n");
+	LOGP(DOML, LOGL_INFO, "Main link established, sending NM Status.\n");
 
-	/* BTS and SITE MGR are EANBLED, BTS is DEPENDENCY */
-	oml_tx_state_changed(&bts->site_mgr.mo);
-	oml_tx_state_changed(&bts->mo);
+	/* BTS SITE MGR becomes Offline (tx SW ACT Report), BTS is DEPENDENCY */
+	osmo_fsm_inst_dispatch(bts->site_mgr.mo.fi, NM_EV_SW_ACT, NULL);
+	osmo_fsm_inst_dispatch(bts->mo.fi, NM_EV_SW_ACT, NULL);
 
 	/* those should all be in DEPENDENCY */
 	oml_tx_state_changed(&bts->gprs.nse.mo);
@@ -278,67 +461,6 @@ int bts_link_estab(struct gsm_bts *bts)
 	return bts_model_oml_estab(bts);
 }
 
-/* RSL link is established, send status report */
-int trx_link_estab(struct gsm_bts_trx *trx)
-{
-	struct e1inp_sign_link *link = trx->rsl_link;
-	uint8_t radio_state = link ?  NM_OPSTATE_ENABLED : NM_OPSTATE_DISABLED;
-	int rc;
-
-	LOGP(DSUM, LOGL_INFO, "RSL link (TRX %02x) state changed to %s, sending Status'.\n",
-		trx->nr, link ? "up" : "down");
-
-	oml_mo_state_chg(&trx->mo, radio_state, NM_AVSTATE_OK);
-
-	if (link)
-		rc = rsl_tx_rf_res(trx);
-	else
-		rc = bts_model_trx_deact_rf(trx);
-	if (rc < 0)
-		oml_fail_rep(OSMO_EVT_MAJ_RSL_FAIL,
-			     link ? "Failed to establish RSL link (%d)" :
-			     "Failed to deactivate RF (%d)", rc);
-	return 0;
-}
-
-/* set the availability of the TRX (used by PHY driver) */
-int trx_set_available(struct gsm_bts_trx *trx, int avail)
-{
-	int tn;
-
-	LOGP(DSUM, LOGL_INFO, "TRX(%d): Setting available = %d\n",
-		trx->nr, avail);
-	if (avail) {
-		/* FIXME: This needs to be sorted out */
-#if 0
-		oml_mo_state_chg(&trx->mo,  NM_OPSTATE_DISABLED, NM_AVSTATE_OFF_LINE);
-		oml_mo_state_chg(&trx->bb_transc.mo, -1, NM_AVSTATE_OFF_LINE);
-		for (tn = 0; tn < ARRAY_SIZE(trx->ts); tn++)
-			oml_mo_state_chg(&trx->ts[tn].mo, NM_OPSTATE_DISABLED, NM_AVSTATE_DEPENDENCY);
-#endif
-	} else {
-		oml_mo_state_chg(&trx->mo,  NM_OPSTATE_DISABLED, NM_AVSTATE_NOT_INSTALLED);
-		oml_mo_state_chg(&trx->bb_transc.mo, -1, NM_AVSTATE_NOT_INSTALLED);
-
-		for (tn = 0; tn < ARRAY_SIZE(trx->ts); tn++)
-			oml_mo_state_chg(&trx->ts[tn].mo, NM_OPSTATE_DISABLED, NM_AVSTATE_NOT_INSTALLED);
-	}
-	return 0;
-}
-
-int lchan_init_lapdm(struct gsm_lchan *lchan)
-{
-	struct lapdm_channel *lc = &lchan->lapdm_ch;
-
-	lapdm_channel_init(lc, LAPDM_MODE_BTS);
-	lapdm_channel_set_flags(lc, LAPDM_ENT_F_POLLING_ONLY);
-	lapdm_channel_set_l1(lc, NULL, lchan);
-	lapdm_channel_set_l3(lc, lapdm_rll_tx_cb, lchan);
-	oml_set_lchan_t200(lchan);
-
-	return 0;
-}
-
 #define CCCH_RACH_RATIO_COMBINED256      (256*1/9)
 #define CCCH_RACH_RATIO_SEPARATE256      (256*10/55)
 
@@ -496,7 +618,7 @@ static int try_merge_imm_ass_rej(struct gsm48_imm_ass_rej *old_rej,
 		return 0;
 
 	/* GSM 08.58, 5.7
-	 * -> The BTS may combine serveral IMM.ASS.REJ messages
+	 * -> The BTS may combine several IMM.ASS.REJ messages
 	 * -> Identical request refs in one message may be squeezed
 	 *
 	 * GSM 04.08, 9.1.20.2
@@ -529,14 +651,14 @@ static int try_merge_imm_ass_rej(struct gsm48_imm_ass_rej *old_rej,
 
 int bts_agch_enqueue(struct gsm_bts *bts, struct msgb *msg)
 {
-	int hard_limit = 1000;
+	int hard_limit = 100;
 	struct gsm48_imm_ass_rej *imm_ass_cmd = msgb_l3(msg);
 
 	if (bts->agch_queue.length > hard_limit) {
-		LOGP(DSUM, LOGL_ERROR,
+		LOGP(DRR, LOGL_ERROR,
 		     "AGCH: too many messages in queue, "
-		     "refusing message type 0x%02x, length = %d/%d\n",
-		     ((struct gsm48_imm_ass *)msgb_l3(msg))->msg_type,
+		     "refusing message type %s, length = %d/%d\n",
+		     gsm48_rr_msg_name(((struct gsm48_imm_ass *)msgb_l3(msg))->msg_type),
 		     bts->agch_queue.length, bts->agch_queue.max_length);
 
 		bts->agch_queue.rejected_msgs++;
@@ -574,7 +696,7 @@ struct msgb *bts_agch_dequeue(struct gsm_bts *bts)
 /*
  * Remove lower prio messages if the queue has grown too long.
  *
- * \return 0 iff the number of messages in the queue would fit into the AGCH
+ * \return 0 if the number of messages in the queue would fit into the AGCH
  *         reserved part of the CCCH.
  */
 static void compact_agch_queue(struct gsm_bts *bts)
@@ -611,11 +733,6 @@ static void compact_agch_queue(struct gsm_bts *bts)
 		struct gsm48_imm_ass *imm_ass_cmd = msgb_l3(msg);
 		int p_drop;
 
-		if (imm_ass_cmd->msg_type != GSM48_MT_RR_IMM_ASS_REJ)
-			return;
-
-		/* IMMEDIATE ASSIGN REJECT */
-
 		p_drop = (bts->agch_queue.length - offs) * slope / max_len;
 
 		if ((random() & 0xffff) >= p_drop)
@@ -623,6 +740,8 @@ static void compact_agch_queue(struct gsm_bts *bts)
 
 		llist_del(&msg->list);
 		bts->agch_queue.length--;
+		rsl_tx_delete_ind(bts, (uint8_t *)imm_ass_cmd, msgb_l3len(msg));
+		rate_ctr_inc2(bts->ctrs, BTS_CTR_AGCH_DELETED);
 		msgb_free(msg);
 
 		bts->agch_queue.dropped_msgs++;
@@ -686,55 +805,157 @@ int bts_supports_cipher(struct gsm_bts *bts, int rsl_cipher)
 	return sup > 0;
 }
 
-int trx_ms_pwr_ctrl_is_osmo(struct gsm_bts_trx *trx)
-{
-	return trx->ms_power_control == 1;
-}
-
 struct gsm_time *get_time(struct gsm_bts *bts)
 {
 	return &bts->gsm_time;
 }
 
-int bts_supports_cm(struct gsm_bts *bts, enum gsm_phys_chan_config pchan,
-		    enum gsm48_chan_mode cm)
+int bts_supports_cm(const struct gsm_bts *bts,
+		    const struct rsl_ie_chan_mode *cm)
 {
-	enum gsm_bts_features feature = _NUM_BTS_FEAT;
+	enum osmo_bts_features feature = _NUM_BTS_FEAT;
+
+	switch (cm->spd_ind) {
+	case RSL_CMOD_SPD_SIGN:
+		/* We assume that signalling support is mandatory,
+		 * there is no BTS_FEAT_* definition to check that. */
+		return 1;
+	case RSL_CMOD_SPD_SPEECH:
+		break;
+	case RSL_CMOD_SPD_DATA:
+	default:
+		return 0;
+	}
 
 	/* Before the requested pchan/cm combination can be checked, we need to
 	 * convert it to a feature identifier we can check */
-	if (pchan == GSM_PCHAN_TCH_F) {
-		switch(cm) {
-		case GSM48_CMODE_SPEECH_V1:
+	switch (cm->chan_rt) {
+	case RSL_CMOD_CRT_OSMO_TCH_VAMOS_Bm:
+		if (!osmo_bts_has_feature(bts->features, BTS_FEAT_VAMOS))
+			return 0;
+		/* fall-through */
+	case RSL_CMOD_CRT_TCH_Bm:
+		switch (cm->chan_rate) {
+		case RSL_CMOD_SP_GSM1:
 			feature	= BTS_FEAT_SPEECH_F_V1;
 			break;
-		case GSM48_CMODE_SPEECH_EFR:
+		case RSL_CMOD_SP_GSM2:
 			feature	= BTS_FEAT_SPEECH_F_EFR;
 			break;
-		case GSM48_CMODE_SPEECH_AMR:
+		case RSL_CMOD_SP_GSM3:
 			feature = BTS_FEAT_SPEECH_F_AMR;
 			break;
 		default:
 			/* Invalid speech codec type => Not supported! */
 			return 0;
 		}
-	} else if (pchan == GSM_PCHAN_TCH_H) {
-		switch(cm) {
-		case GSM48_CMODE_SPEECH_V1:
+		break;
+
+	case RSL_CMOD_CRT_OSMO_TCH_VAMOS_Lm:
+		if (!osmo_bts_has_feature(bts->features, BTS_FEAT_VAMOS))
+			return 0;
+		/* fall-through */
+	case RSL_CMOD_CRT_TCH_Lm:
+		switch (cm->chan_rate) {
+		case RSL_CMOD_SP_GSM1:
 			feature	= BTS_FEAT_SPEECH_H_V1;
 			break;
-		case GSM48_CMODE_SPEECH_AMR:
+		case RSL_CMOD_SP_GSM3:
 			feature = BTS_FEAT_SPEECH_H_AMR;
 			break;
 		default:
 			/* Invalid speech codec type => Not supported! */
 			return 0;
 		}
+		break;
+
+	default:
+		LOGP(DRSL, LOGL_ERROR,
+		     "Unhandled RSL channel type=0x%02x/rate=0x%02x\n",
+		     cm->chan_rt, cm->chan_rate);
+		return 0;
 	}
 
 	/* Check if the feature is supported by this BTS */
-	if (gsm_bts_has_feature(bts, feature))
+	if (osmo_bts_has_feature(bts->features, feature))
 		return 1;
 
 	return 0;
 }
+
+/* return the gsm_lchan for the CBCH (if it exists at all) */
+struct gsm_lchan *gsm_bts_get_cbch(struct gsm_bts *bts)
+{
+	struct gsm_bts_trx *trx = bts->c0;
+
+	/* According to 3GPP TS 45.002, table 3, CBCH can be allocated
+	 * either on C0/TS0 (CCCH+SDCCH4) or on C0..n/TS0..3 (SDCCH/8). */
+	if (trx->ts[0].pchan == GSM_PCHAN_CCCH_SDCCH4_CBCH)
+		return &trx->ts[0].lchan[2]; /* C0/TS0 */
+
+	llist_for_each_entry(trx, &bts->trx_list, list) { /* C0..n */
+		unsigned int tn;
+		for (tn = 0; tn <= 3; tn++) { /* TS0..3 */
+			struct gsm_bts_trx_ts *ts = &trx->ts[tn];
+			if (ts->pchan == GSM_PCHAN_SDCCH8_SACCH8C_CBCH)
+				return &ts->lchan[2];
+		}
+	}
+
+	return NULL;
+}
+
+/* BCCH carrier power reduction (see 3GPP TS 45.008, section 7.1) */
+int bts_set_c0_pwr_red(struct gsm_bts *bts, const uint8_t red)
+{
+	struct gsm_bts_trx *c0 = bts->c0;
+	unsigned int tn;
+
+	if (!osmo_bts_has_feature(bts->features, BTS_FEAT_BCCH_POWER_RED)) {
+		LOGPTRX(c0, DRSL, LOGL_ERROR, "BCCH carrier power reduction "
+			"is not supported by this BTS model\n");
+		return -ENOTSUP;
+	}
+
+	if (red > 6 || red % 2 != 0) {
+		LOGPTRX(c0, DRSL, LOGL_ERROR, "BCCH carrier power reduction "
+			"value (%u dB) is incorrect or out of range\n", red);
+		return -EINVAL;
+	}
+
+	LOGPTRX(c0, DRSL, LOGL_NOTICE, "BCCH carrier power reduction: "
+		"%u dB (%s)\n", red, red ? "enabled" : "disabled");
+
+	/* Timeslot 0 is always transmitting BCCH/CCCH */
+	c0->ts[0].c0_power_red_db = 0;
+
+	for (tn = 1; tn < ARRAY_SIZE(c0->ts); tn++) {
+		struct gsm_bts_trx_ts *ts = &c0->ts[tn];
+		struct gsm_bts_trx_ts *prev = ts - 1;
+
+		switch (ts_pchan(ts)) {
+		/* Not allowed on CCCH/BCCH */
+		case GSM_PCHAN_CCCH:
+			/* Preceeding timeslot shall not exceed 2 dB */
+			if (prev->c0_power_red_db > 0)
+				prev->c0_power_red_db = 2;
+			/* fall-through */
+		/* Not recommended on SDCCH/8 */
+		case GSM_PCHAN_SDCCH8_SACCH8C:
+		case GSM_PCHAN_SDCCH8_SACCH8C_CBCH:
+			ts->c0_power_red_db = 0;
+			break;
+		default:
+			ts->c0_power_red_db = red;
+			break;
+		}
+	}
+
+	/* Timeslot 7 is always preceding BCCH/CCCH */
+	if (c0->ts[7].c0_power_red_db > 0)
+		c0->ts[7].c0_power_red_db = 2;
+
+	bts->c0_power_red_db = red;
+
+	return 0;
+}
