@@ -27,6 +27,7 @@
 #include <osmo-bts/gsm_data.h>
 #include <osmo-bts/rsl.h>
 #include <osmo-bts/paging.h>
+#include <osmo-bts/bts.h>
 
 static void reset_load_counters(struct gsm_bts *bts)
 {
@@ -40,6 +41,10 @@ static void load_timer_cb(void *data)
 	struct gsm_bts *bts = data;
 	unsigned int pch_percent, rach_percent;
 
+	/* It makes no sense to send Load Indication if CCCH is still disabled...*/
+	if (bts->c0->ts[0].mo.nm_state.operational != NM_OPSTATE_ENABLED)
+		goto retry_later;
+
 	/* compute percentages */
 	if (bts->load.ccch.pch_total == 0)
 		pch_percent = 0;
@@ -72,6 +77,7 @@ static void load_timer_cb(void *data)
 					  bts->load.rach.access);
 	}
 
+retry_later:
 	reset_load_counters(bts);
 
 	/* re-schedule the timer */
@@ -93,3 +99,8 @@ void load_timer_stop(struct gsm_bts *bts)
 {
 	osmo_timer_del(&bts->load.ccch.timer);
 }
+
+bool load_timer_is_running(const struct gsm_bts *bts)
+{
+	return osmo_timer_pending(&bts->load.ccch.timer);
+}
