@@ -16,10 +16,17 @@
 #include <osmo-bts/power_control.h>
 #include <osmo-bts/ta_control.h>
 
-/* Tables as per TS 45.008 Section 8.3 */
-static const uint8_t ts45008_83_tch_f[] = { 52, 53, 54, 55, 56, 57, 58, 59 };
-static const uint8_t ts45008_83_tch_hs0[] = { 0, 2, 4, 6, 52, 54, 56, 58 };
-static const uint8_t ts45008_83_tch_hs1[] = { 14, 16, 18, 20, 66, 68, 70, 72 };
+/* Active TDMA frame subset for TCH/H in DTX mode (see 3GPP TS 45.008 Section 8.3).
+ * This mapping is used to determine if a L2 block starting at the given TDMA FN
+ * belongs to the SUB set and thus shall always be transmitted in DTX mode. */
+static const uint8_t ts45008_dtx_tchh_fn_map[104] = {
+	/* TCH/H(0): 0, 2, 4, 6, 52, 54, 56, 58 */
+	[0]  = 1, /* block { 0,  2,  4,  6} */
+	[52] = 1, /* block {52, 54, 56, 58} */
+	/* TCH/H(1): 14, 16, 18, 20, 66, 68, 70, 72 */
+	[14] = 1, /* block {14, 16, 18, 20} */
+	[66] = 1, /* block {66, 68, 70, 72} */
+};
 
 /* In cases where we less measurements than we expect we must assume that we
  * just did not receive the block because it was lost due to bad channel
@@ -31,22 +38,11 @@ static const uint8_t ts45008_83_tch_hs1[] = { 14, 16, 18, 20, 66, 68, 70, 72 };
 static const struct bts_ul_meas measurement_dummy = {
 	.ber10k = MEASUREMENT_DUMMY_BER,
 	.ta_offs_256bits = 0,
-	.c_i = 0,
+	.ci_cb = 0,
 	.is_sub = 0,
 	.inv_rssi = MEASUREMENT_DUMMY_IRSSI
 };
 
-/* find out if an array contains a given key as element */
-#define ARRAY_CONTAINS(arr, val) array_contains(arr, ARRAY_SIZE(arr), val)
-static bool array_contains(const uint8_t *arr, unsigned int len, uint8_t val) {
-	int i;
-	for (i = 0; i < len; i++) {
-		if (arr[i] == val)
-			return true;
-	}
-	return false;
-}
-
 /* Decide if a given frame number is part of the "-SUB" measurements (true) or not (false)
  * (this function is only used internally, it is public to call it from unit-tests) */
 bool ts45008_83_is_sub(struct gsm_lchan *lchan, uint32_t fn)
@@ -56,18 +52,20 @@ bool ts45008_83_is_sub(struct gsm_lchan *lchan, uint32_t fn)
 	/* See TS 45.008 Sections 8.3 and 8.4 for a detailed descriptions of the rules
 	 * implemented here. We only implement the logic for Voice, not CSD */
 
+	/* AMR is special, SID frames may be scheduled dynamically at any time */
+	if (lchan->tch_mode == GSM48_CMODE_SPEECH_AMR)
+		return false;
+
 	switch (lchan->type) {
 	case GSM_LCHAN_TCH_F:
 		switch (lchan->tch_mode) {
 		case GSM48_CMODE_SPEECH_V1:
 		case GSM48_CMODE_SPEECH_EFR:
-			if (trx_sched_is_sacch_fn(lchan->ts, fn, true))
-				return true;
-			if (ARRAY_CONTAINS(ts45008_83_tch_f, fn104))
-				return true;
-			break;
-		case GSM48_CMODE_SPEECH_AMR:
-			if (trx_sched_is_sacch_fn(lchan->ts, fn, true))
+			/* Active TDMA frame subset for TCH/F: 52, 53, 54, 55, 56, 57, 58, 59.
+			 * There is only one *complete* block in this subset starting at FN=52.
+			 * Incomplete blocks {... 52, 53, 54, 55} and {56, 57, 58, 59 ...}
+			 * contain only 50% of the useful bits (partial SID) and thus ~50% BER. */
+			if (fn104 == 52)
 				return true;
 			break;
 		case GSM48_CMODE_SIGN:
@@ -83,23 +81,7 @@ bool ts45008_83_is_sub(struct gsm_lchan *lchan, uint32_t fn)
 	case GSM_LCHAN_TCH_H:
 		switch (lchan->tch_mode) {
 		case GSM48_CMODE_SPEECH_V1:
-			if (trx_sched_is_sacch_fn(lchan->ts, fn, true))
-				return true;
-			switch (lchan->nr) {
-			case 0:
-				if (ARRAY_CONTAINS(ts45008_83_tch_hs0, fn104))
-					return true;
-				break;
-			case 1:
-				if (ARRAY_CONTAINS(ts45008_83_tch_hs1, fn104))
-					return true;
-				break;
-			default:
-				OSMO_ASSERT(0);
-			}
-			break;
-		case GSM48_CMODE_SPEECH_AMR:
-			if (trx_sched_is_sacch_fn(lchan->ts, fn, true))
+			if (ts45008_dtx_tchh_fn_map[fn104])
 				return true;
 			break;
 		case GSM48_CMODE_SIGN:
@@ -346,10 +328,11 @@ int lchan_new_ul_meas(struct gsm_lchan *lchan,
 	if (!ulm->is_sub)
 		dest->is_sub = ts45008_83_is_sub(lchan, fn);
 
-	DEBUGPFN(DMEAS, fn, "%s adding measurement (ber10k=%u, ta_offs=%d, ci_cB=%d, is_sub=%u, rssi=-%u), num_ul_meas=%d, fn_mod=%u\n",
-		 gsm_lchan_name(lchan), ulm->ber10k, ulm->ta_offs_256bits,
-		 ulm->c_i, dest->is_sub, ulm->inv_rssi, lchan->meas.num_ul_meas,
-		 fn_mod);
+	DEBUGPFN(DMEAS, fn, "%s adding a %s measurement (ber10k=%u, ta_offs=%d, ci_cB=%d, rssi=-%u), num_ul_meas=%d, fn_mod=%u\n",
+		 gsm_lchan_name(lchan),
+		 dest->is_sub ? "SUB" : "FULL",
+		 ulm->ber10k, ulm->ta_offs_256bits, ulm->ci_cb, ulm->inv_rssi,
+		 lchan->meas.num_ul_meas, fn_mod);
 
 	lchan->meas.last_fn = fn;
 
@@ -439,16 +422,16 @@ static unsigned int lchan_meas_sub_num_expected(const struct gsm_lchan *lchan)
 			/* 1 block SACCH, 24 blocks TCH (see note 1) */
 			return 25;
 		} else {
-			/* 1 block SACCH, 2 blocks TCH */
-			return 3;
+			/* 1 block SACCH, 1 block TCH */
+			return 2;
 		}
 	case GSM_PCHAN_TCH_H:
 		if (lchan->tch_mode == GSM48_CMODE_SIGN) {
 			/* 1 block SACCH, 12 blocks TCH (see ynote 1) */
 			return 13;
 		} else {
-			/* 1 block SACCH, 4 blocks TCH */
-			return 5;
+			/* 1 block SACCH, 2 blocks TCH */
+			return 3;
 		}
 	case GSM_PCHAN_SDCCH8_SACCH8C:
 	case GSM_PCHAN_SDCCH8_SACCH8C_CBCH:
@@ -631,13 +614,13 @@ int lchan_meas_check_compute(struct gsm_lchan *lchan, uint32_t fn)
 			m = &lchan->meas.uplink[i + num_ul_meas_excess];
 			if (m->is_sub) {
 				irssi_sub_sum += m->inv_rssi;
-				ci_sub_sum += m->c_i;
+				ci_sub_sum += m->ci_cb;
 				num_meas_sub_actual++;
 				is_sub = true;
 			}
 			irssi_full_sum += m->inv_rssi;
 			ta256b_sum += m->ta_offs_256bits;
-			ci_full_sum += m->c_i;
+			ci_full_sum += m->ci_cb;
 
 			num_ul_meas_actual++;
 		} else {
