@@ -173,7 +173,8 @@ static inline void add_bts_attrs(struct msgb *msg, const struct gsm_bts *bts)
 /* Add BTS features as 3GPP TS 52.021 §9.4.30 Manufacturer Id */
 static inline void add_bts_feat(struct msgb *msg, const struct gsm_bts *bts)
 {
-	msgb_tl16v_put(msg, NM_ATT_MANUF_ID, _NUM_BTS_FEAT/8 + 1, bts->_features_data);
+	unsigned int len = OSMO_BYTES_FOR_BITS(_NUM_BTS_FEAT);
+	msgb_tl16v_put(msg, NM_ATT_MANUF_ID, len, bts->features->data);
 }
 
 static inline void add_trx_attr(struct msgb *msg, const struct gsm_bts_trx *trx)
@@ -313,6 +314,7 @@ static int oml_tx_attr_resp(const struct gsm_abis_mo *mo,
 int oml_tx_state_changed(const struct gsm_abis_mo *mo)
 {
 	struct msgb *nmsg;
+	uint8_t avail_state;
 
 	nmsg = oml_msgb_alloc();
 	if (!nmsg)
@@ -322,7 +324,8 @@ int oml_tx_state_changed(const struct gsm_abis_mo *mo)
 	msgb_tv_put(nmsg, NM_ATT_OPER_STATE, mo->nm_state.operational);
 
 	/* 9.4.7 Availability Status */
-	msgb_tl16v_put(nmsg, NM_ATT_AVAIL_STATUS, 1, &mo->nm_state.availability);
+	avail_state = (uint8_t) mo->nm_state.availability;
+	msgb_tl16v_put(nmsg, NM_ATT_AVAIL_STATUS, 1, &avail_state);
 
 	/* 9.4.4 Administrative Status -- not in spec but also sent by nanobts */
 	msgb_tv_put(nmsg, NM_ATT_ADM_STATE, mo->nm_state.administrative);
@@ -337,12 +340,13 @@ void oml_mo_state_init(struct gsm_abis_mo *mo, int op_state, int avail_state)
 	mo->nm_state.operational = op_state;
 }
 
-int oml_mo_state_chg(struct gsm_abis_mo *mo, int op_state, int avail_state)
+int oml_mo_state_chg(struct gsm_abis_mo *mo, int op_state, int avail_state, int adm_state)
 {
 	int rc = 0;
 
 	if ((op_state != -1 && mo->nm_state.operational != op_state) ||
-	    (avail_state != -1 && mo->nm_state.availability != avail_state)) {
+	    (avail_state != -1 && mo->nm_state.availability != avail_state) ||
+	    (adm_state != -1 && mo->nm_state.administrative != adm_state)) {
 		if (avail_state != -1) {
 			LOGP(DOML, LOGL_INFO, "%s AVAIL STATE %s -> %s\n",
 				gsm_abis_mo_name(mo),
@@ -351,13 +355,25 @@ int oml_mo_state_chg(struct gsm_abis_mo *mo, int op_state, int avail_state)
 			mo->nm_state.availability = avail_state;
 		}
 		if (op_state != -1) {
+			struct nm_statechg_signal_data nsd;
 			LOGP(DOML, LOGL_INFO, "%s OPER STATE %s -> %s\n",
 				gsm_abis_mo_name(mo),
 				abis_nm_opstate_name(mo->nm_state.operational),
 				abis_nm_opstate_name(op_state));
+			nsd.mo = mo;
+			nsd.old_state = mo->nm_state.operational;
+			nsd.new_state = op_state;
 			mo->nm_state.operational = op_state;
-			osmo_signal_dispatch(SS_GLOBAL, S_NEW_OP_STATE, NULL);
+			osmo_signal_dispatch(SS_GLOBAL, S_NEW_OP_STATE, &nsd);
 		}
+		if (adm_state != -1) {
+			LOGP(DOML, LOGL_INFO, "%s ADMIN STATE %s -> %s\n",
+				gsm_abis_mo_name(mo),
+				abis_nm_admin_name(mo->nm_state.administrative),
+				abis_nm_admin_name(adm_state));
+			mo->nm_state.administrative = adm_state;
+		}
+
 
 		/* send state change report */
 		rc = oml_tx_state_changed(mo);
@@ -421,23 +437,15 @@ int oml_mo_opstart_nack(const struct gsm_abis_mo *mo, uint8_t nack_cause)
 	return oml_mo_fom_ack_nack(mo, NM_MT_OPSTART, nack_cause);
 }
 
-/* Send an ACK or NACK response for 'msg' to BSC, deriving message
- * type, obj class, obj inst from 'msg' and copying all attributes
- * contained in 'msg'. ACK is sent if cause == 0; NACK otherwise */
-int oml_fom_ack_nack(struct msgb *old_msg, uint8_t cause)
+/* Send an ACK or NACK response to BSC for the given OML message,
+ * reusing it.  ACK is sent if cause == 0; NACK otherwise. */
+int oml_fom_ack_nack(struct msgb *msg, uint8_t cause)
 {
-	struct msgb *msg;
 	struct abis_om_fom_hdr *foh;
 
-	msg = msgb_copy(old_msg, "OML_fom_ack_nack");
-	if (!msg)
-		return -ENOMEM;
-
-	/* remove any l2/l1 that may be present in copy */
+	/* remove any l2/l1 that may be already present */
 	msgb_pull_to_l2(msg);
 
-	msg->trx = old_msg->trx;
-
 	foh = (struct abis_om_fom_hdr *) msg->l3h;
 
 	/* alter message type */
@@ -447,16 +455,21 @@ int oml_fom_ack_nack(struct msgb *old_msg, uint8_t cause)
 		foh->msg_type += 2; /* nack */
 		/* add cause */
 		msgb_tv_put(msg, NM_ATT_NACK_CAUSES, cause);
-		/* update the length as we just made the message larger */
-		struct abis_om_hdr *omh = (struct abis_om_hdr *) msgb_l2(msg);
-		omh->length = msgb_l3len(msg);
 	} else {
 		LOGPFOH(DOML, LOGL_DEBUG, foh, "Sending FOM ACK.\n");
 		foh->msg_type++; /* ack */
 	}
 
+	/* ensure that the message length is up to date */
+	struct abis_om_hdr *omh = (struct abis_om_hdr *) msgb_l2(msg);
+	omh->length = msgb_l3len(msg);
+
 	/* we cannot use oml_send_msg() as we already have the OML header */
-	return abis_oml_sendmsg(msg);
+	if (abis_oml_sendmsg(msg) != 0)
+		LOGPFOH(DOML, LOGL_ERROR, foh, "Failed to send ACK/NACK\n");
+
+	/* msgb was reused, do not free() */
+	return 1;
 }
 
 /*
@@ -472,7 +485,6 @@ int oml_mo_tx_sw_act_rep(const struct gsm_abis_mo *mo)
 	if (!nmsg)
 		return -ENOMEM;
 
-	msgb_put(nmsg, sizeof(struct abis_om_fom_hdr));
 	return oml_mo_send_msg(mo, nmsg, NM_MT_SW_ACTIVATED_REP);
 }
 
@@ -550,7 +562,7 @@ static int oml_rx_set_bts_attr(struct gsm_bts *bts, struct msgb *msg)
 	/* Test for globally unsupported stuff here */
 	if (TLVP_PRES_LEN(&tp, NM_ATT_BCCH_ARFCN, 2)) {
 		uint16_t arfcn = ntohs(tlvp_val16_unal(&tp, NM_ATT_BCCH_ARFCN));
-		if (arfcn > 1024) {
+		if (arfcn >= 1024) { /* 0 .. 1023 (1024 channels total) */
 			oml_tx_failure_event_rep(&bts->mo, NM_SEVER_MAJOR, OSMO_EVT_WARN_SW_WARN,
 						 "Given ARFCN %u is not supported",
 						 arfcn);
@@ -568,6 +580,7 @@ static int oml_rx_set_bts_attr(struct gsm_bts *bts, struct msgb *msg)
 
 	/* merge existing BTS attributes with new attributes */
 	tp_merged = osmo_tlvp_copy(bts->mo.nm_attr, bts);
+	talloc_set_name_const(tp_merged, "oml_bts_attr");
 	osmo_tlvp_merge(tp_merged, &tp);
 
 	/* Ask BTS driver to validate new merged attributes */
@@ -586,8 +599,8 @@ static int oml_rx_set_bts_attr(struct gsm_bts *bts, struct msgb *msg)
 	/* 9.4.25 Interference Level Boundaries */
 	if (TLVP_PRES_LEN(&tp, NM_ATT_INTERF_BOUND, 6)) {
 		payload = TLVP_VAL(&tp, NM_ATT_INTERF_BOUND);
-		for (i = 0; i < 6; i++) {
-			int16_t boundary = *payload;
+		for (i = 0; i < ARRAY_SIZE(bts->interference.boundary); i++) {
+			const int16_t boundary = payload[i];
 			bts->interference.boundary[i] = -1 * boundary;
 		}
 	}
@@ -603,12 +616,16 @@ static int oml_rx_set_bts_attr(struct gsm_bts *bts, struct msgb *msg)
 		case 0xFF: /* Osmocom specific Extension of TS 12.21 */
 			LOGPFOH(DOML, LOGL_NOTICE, foh, "WARNING: Radio Link Timeout "
 				"explicitly disabled, only use this for lab testing!\n");
-			bts->radio_link_timeout = -1;
+			bts->radio_link_timeout.oml = -1;
+			if (!bts->radio_link_timeout.vty_override)
+				bts->radio_link_timeout.current = bts->radio_link_timeout.oml;
 			break;
 		case 0x01: /* Based on uplink SACCH (radio link timeout) */
 			if (TLVP_LEN(&tp, NM_ATT_CONN_FAIL_CRIT) >= 2 &&
 			    val[1] >= 4 && val[1] <= 64) {
-				bts->radio_link_timeout = val[1];
+				bts->radio_link_timeout.oml = val[1];
+				if (!bts->radio_link_timeout.vty_override)
+					bts->radio_link_timeout.current = bts->radio_link_timeout.oml;
 				break;
 			}
 			/* fall-through */
@@ -658,7 +675,10 @@ static int oml_rx_set_bts_attr(struct gsm_bts *bts, struct msgb *msg)
 	/* 9.4.11 CCCH Load Indication Period */
 	if (TLVP_PRES_LEN(&tp, NM_ATT_CCCH_L_I_P, 1)) {
 		bts->load.ccch.load_ind_period = *TLVP_VAL(&tp, NM_ATT_CCCH_L_I_P);
-		load_timer_start(bts);
+		if (load_timer_is_running(bts)) {
+			load_timer_stop(bts);
+			load_timer_start(bts);
+		}
 	}
 
 	/* 9.4.44 RACH Busy Threshold */
@@ -716,8 +736,9 @@ static int oml_rx_set_radio_attr(struct gsm_bts_trx *trx, struct msgb *msg)
 		return oml_fom_ack_nack(msg, NM_NACK_INCORR_STRUCT);
 	}
 
-	/* merge existing BTS attributes with new attributes */
-	tp_merged = osmo_tlvp_copy(trx->mo.nm_attr, trx->bts);
+	/* merge existing TRX attributes with new attributes */
+	tp_merged = osmo_tlvp_copy(trx->mo.nm_attr, trx);
+	talloc_set_name_const(tp_merged, "oml_trx_attr");
 	osmo_tlvp_merge(tp_merged, &tp);
 
 	/* Ask BTS driver to validate new merged attributes */
@@ -727,7 +748,7 @@ static int oml_rx_set_radio_attr(struct gsm_bts_trx *trx, struct msgb *msg)
 		return oml_fom_ack_nack(msg, -rc);
 	}
 
-	/* Success: replace old BTS attributes with new */
+	/* Success: replace old TRX attributes with new */
 	talloc_free(trx->mo.nm_attr);
 	trx->mo.nm_attr = tp_merged;
 
@@ -773,7 +794,7 @@ static int oml_rx_set_radio_attr(struct gsm_bts_trx *trx, struct msgb *msg)
 		memcpy(&_value, value, 2);
 		arfcn = ntohs(_value);
 		value += 2;
-		if (arfcn > 1024) {
+		if (arfcn >= 1024) { /* 0 .. 1023 (1024 channels total) */
 			oml_tx_failure_event_rep(&trx->bts->mo, NM_SEVER_MAJOR, OSMO_EVT_WARN_SW_WARN,
 						 "Given ARFCN %u is unsupported", arfcn);
 			LOGPFOH(DOML, LOGL_NOTICE, foh, "Given ARFCN %u is unsupported.\n", arfcn);
@@ -786,9 +807,12 @@ static int oml_rx_set_radio_attr(struct gsm_bts_trx *trx, struct msgb *msg)
 	return bts_model_apply_oml(trx->bts, msg, tp_merged, NM_OC_RADIO_CARRIER, trx);
 }
 
-static int conf_lchans(struct gsm_bts_trx_ts *ts)
+static int handle_chan_comb(struct gsm_bts_trx_ts *ts, const uint8_t comb)
 {
-	enum gsm_phys_chan_config pchan = ts->pchan;
+	enum gsm_phys_chan_config pchan;
+
+	pchan = abis_nm_pchan4chcomb(comb);
+	ts->pchan = pchan;
 
 	/* RSL_MT_IPAC_PDCH_ACT style dyn PDCH */
 	if (pchan == GSM_PCHAN_TCH_F_PDCH)
@@ -796,7 +820,7 @@ static int conf_lchans(struct gsm_bts_trx_ts *ts)
 						    : GSM_PCHAN_TCH_F;
 
 	/* Osmocom RSL CHAN ACT style dyn TS */
-	if (pchan == GSM_PCHAN_TCH_F_TCH_H_PDCH) {
+	if (pchan == GSM_PCHAN_OSMO_DYN) {
 		pchan = ts->dyn.pchan_is;
 
 		/* If the dyn TS doesn't have a pchan yet, do nothing. */
@@ -807,62 +831,65 @@ static int conf_lchans(struct gsm_bts_trx_ts *ts)
 	return conf_lchans_as_pchan(ts, pchan);
 }
 
+static inline void lchans_type_set(struct gsm_bts_trx_ts *ts,
+				   enum gsm_chan_t lchan_type,
+				   unsigned int num_lchans)
+{
+	unsigned int i;
+
+	for (i = 0; i < num_lchans; i++)
+		ts->lchan[i].type = lchan_type;
+}
+
 int conf_lchans_as_pchan(struct gsm_bts_trx_ts *ts,
 			 enum gsm_phys_chan_config pchan)
 {
-	struct gsm_lchan *lchan;
-	unsigned int i;
+	/* Initialize all lchans with GSM_LCHAN_NONE first */
+	lchans_type_set(ts, GSM_LCHAN_NONE, ARRAY_SIZE(ts->lchan));
 
 	switch (pchan) {
 	case GSM_PCHAN_CCCH_SDCCH4_CBCH:
-		/* fallthrough */
 	case GSM_PCHAN_CCCH_SDCCH4:
-		for (i = 0; i < 4; i++) {
-			lchan = &ts->lchan[i];
-			if (pchan == GSM_PCHAN_CCCH_SDCCH4_CBCH
-			    && i == 2) {
-				lchan->type = GSM_LCHAN_CBCH;
-			} else {
-				lchan->type = GSM_LCHAN_SDCCH;
-			}
-		}
+		lchans_type_set(ts, GSM_LCHAN_SDCCH, 4);
+		if (pchan == GSM_PCHAN_CCCH_SDCCH4_CBCH)
+			ts->lchan[2].type = GSM_LCHAN_CBCH;
 		/* fallthrough */
 	case GSM_PCHAN_CCCH:
-		lchan = &ts->lchan[CCCH_LCHAN];
-		lchan->type = GSM_LCHAN_CCCH;
+		ts->lchan[CCCH_LCHAN].type = GSM_LCHAN_CCCH;
 		break;
 	case GSM_PCHAN_TCH_F:
-		lchan = &ts->lchan[0];
-		lchan->type = GSM_LCHAN_TCH_F;
+		if (ts->vamos.peer != NULL) { /* VAMOS: enable shadow lchans */
+			lchans_type_set(ts->vamos.peer, GSM_LCHAN_TCH_F, 1);
+			ts->vamos.peer->pchan = GSM_PCHAN_TCH_F;
+		}
+		lchans_type_set(ts, GSM_LCHAN_TCH_F, 1);
 		break;
 	case GSM_PCHAN_TCH_H:
-		for (i = 0; i < 2; i++) {
-			lchan = &ts->lchan[i];
-			lchan->type = GSM_LCHAN_TCH_H;
+		if (ts->vamos.peer != NULL) { /* VAMOS: enable shadow lchans */
+			lchans_type_set(ts->vamos.peer, GSM_LCHAN_TCH_H, 2);
+			ts->vamos.peer->pchan = GSM_PCHAN_TCH_H;
 		}
+		lchans_type_set(ts, GSM_LCHAN_TCH_H, 2);
 		break;
 	case GSM_PCHAN_SDCCH8_SACCH8C_CBCH:
-		/* fallthrough */
 	case GSM_PCHAN_SDCCH8_SACCH8C:
-		for (i = 0; i < 8; i++) {
-			lchan = &ts->lchan[i];
-			if (pchan == GSM_PCHAN_SDCCH8_SACCH8C_CBCH
-			    && i == 2) {
-				lchan->type = GSM_LCHAN_CBCH;
-			} else {
-				lchan->type = GSM_LCHAN_SDCCH;
-			}
-		}
+		lchans_type_set(ts, GSM_LCHAN_SDCCH, 8);
+		if (pchan == GSM_PCHAN_SDCCH8_SACCH8C_CBCH)
+			ts->lchan[2].type = GSM_LCHAN_CBCH;
 		break;
 	case GSM_PCHAN_PDCH:
-		lchan = &ts->lchan[0];
-		lchan->type = GSM_LCHAN_PDTCH;
+		if (ts->vamos.peer != NULL) { /* VAMOS: disable shadow lchans */
+			lchans_type_set(ts->vamos.peer, GSM_LCHAN_NONE, 1);
+			ts->vamos.peer->pchan = GSM_PCHAN_NONE;
+		}
+		lchans_type_set(ts, GSM_LCHAN_PDTCH, 1);
 		break;
 	default:
 		LOGP(DOML, LOGL_ERROR, "Unknown/unhandled PCHAN type: %u %s\n",
 		     ts->pchan, gsm_pchan_name(ts->pchan));
 		return -NM_NACK_PARAM_RANGE;
 	}
+
 	return 0;
 }
 
@@ -872,7 +899,7 @@ static int oml_rx_set_chan_attr(struct gsm_bts_trx_ts *ts, struct msgb *msg)
 	struct abis_om_fom_hdr *foh = msgb_l3(msg);
 	struct gsm_bts *bts = ts->trx->bts;
 	struct tlv_parsed tp, *tp_merged;
-	int rc;
+	int rc, i;
 
 	DEBUGPFOH(DOML, foh, "Rx SET CHAN ATTR\n");
 
@@ -883,11 +910,39 @@ static int oml_rx_set_chan_attr(struct gsm_bts_trx_ts *ts, struct msgb *msg)
 		return oml_fom_ack_nack(msg, NM_NACK_INCORR_STRUCT);
 	}
 
-	/* 9.4.21 HSN... */
-	/* 9.4.27 MAIO */
+	/* Check frequency hopping parameters (HSN, MAIO, ARFCN list) */
 	if (TLVP_PRESENT(&tp, NM_ATT_HSN) || TLVP_PRESENT(&tp, NM_ATT_MAIO)) {
-		LOGPFOH(DOML, LOGL_NOTICE, foh, "SET CHAN ATTR: Frequency hopping not supported.\n");
-		return oml_fom_ack_nack(msg, NM_NACK_SPEC_IMPL_NOTSUPP);
+		if (!osmo_bts_has_feature(bts->features, BTS_FEAT_HOPPING)) {
+			LOGPFOH(DOML, LOGL_ERROR, foh, "SET CHAN ATTR: Frequency hopping not supported.\n");
+			return oml_fom_ack_nack(msg, NM_NACK_SPEC_IMPL_NOTSUPP);
+		}
+
+		if (!TLVP_PRES_LEN(&tp, NM_ATT_HSN, 1) || !TLVP_PRES_LEN(&tp, NM_ATT_MAIO, 1)) {
+			LOGPFOH(DOML, LOGL_ERROR, foh, "SET CHAN ATTR: HSN and/or MAIO is missing: "
+				"hsn=%u, maio=%u\n", TLVP_LEN(&tp, NM_ATT_HSN), TLVP_LEN(&tp, NM_ATT_MAIO));
+			return oml_fom_ack_nack(msg, NM_NACK_ATTRLIST_INCONSISTENT);
+		}
+
+		if (!TLVP_PRES_LEN(&tp, NM_ATT_ARFCN_LIST, 2)) { /* At least one ARFCN */
+			LOGPFOH(DOML, LOGL_ERROR, foh, "SET CHAN ATTR: ARFCN list is missing\n");
+			return oml_fom_ack_nack(msg, NM_NACK_ATTRLIST_INCONSISTENT);
+		}
+
+		if (TLVP_LEN(&tp, NM_ATT_ARFCN_LIST) > sizeof(ts->hopping.arfcn_list)) {
+			LOGPFOH(DOML, LOGL_ERROR, foh, "SET CHAN ATTR: ARFCN list is too long\n");
+			return oml_fom_ack_nack(msg, NM_NACK_ATTRLIST_INCONSISTENT);
+		} else if (TLVP_LEN(&tp, NM_ATT_ARFCN_LIST) % 2 != 0) {
+			LOGPFOH(DOML, LOGL_ERROR, foh, "SET CHAN ATTR: ARFCN list has odd length\n");
+			return oml_fom_ack_nack(msg, NM_NACK_ATTRLIST_INCONSISTENT);
+		}
+
+		ts->hopping.enabled = true;
+		ts->hopping.hsn = *TLVP_VAL(&tp, NM_ATT_HSN);
+		ts->hopping.maio = *TLVP_VAL(&tp, NM_ATT_MAIO);
+
+		ts->hopping.arfcn_num = TLVP_LEN(&tp, NM_ATT_ARFCN_LIST) / sizeof(uint16_t);
+		for (i = 0; i < ts->hopping.arfcn_num; i++)
+			ts->hopping.arfcn_list[i] = osmo_load16be(TLVP_VAL(&tp, NM_ATT_ARFCN_LIST) + i * 2);
 	}
 
 	/* 9.4.52 Starting Time */
@@ -896,8 +951,9 @@ static int oml_rx_set_chan_attr(struct gsm_bts_trx_ts *ts, struct msgb *msg)
 		return oml_fom_ack_nack(msg, NM_NACK_SPEC_IMPL_NOTSUPP);
 	}
 
-	/* merge existing BTS attributes with new attributes */
-	tp_merged = osmo_tlvp_copy(ts->mo.nm_attr, bts);
+	/* merge existing CHAN attributes with new attributes */
+	tp_merged = osmo_tlvp_copy(ts->mo.nm_attr, ts->trx);
+	talloc_set_name_const(tp_merged, "oml_chan_attr");
 	osmo_tlvp_merge(tp_merged, &tp);
 
 	/* Call into BTS driver to check attribute values */
@@ -909,18 +965,16 @@ static int oml_rx_set_chan_attr(struct gsm_bts_trx_ts *ts, struct msgb *msg)
 		return oml_fom_ack_nack(msg, -rc);
 	}
 
-	/* Success: replace old BTS attributes with new */
+	/* Success: replace old CHAN attributes with new */
 	talloc_free(ts->mo.nm_attr);
 	ts->mo.nm_attr = tp_merged;
 
 	/* 9.4.13 Channel Combination */
 	if (TLVP_PRES_LEN(&tp, NM_ATT_CHAN_COMB, 1)) {
-		uint8_t comb = *TLVP_VAL(&tp, NM_ATT_CHAN_COMB);
-		ts->pchan = abis_nm_pchan4chcomb(comb);
-		rc = conf_lchans(ts);
-		if (rc < 0) {
+		const uint8_t comb = *TLVP_VAL(&tp, NM_ATT_CHAN_COMB);
+		if ((rc = handle_chan_comb(ts, comb)) != 0) {
 			LOGPFOH(DOML, LOGL_ERROR, foh, "SET CHAN ATTR: invalid Chan Comb 0x%x"
-				" (pchan=%s, conf_lchans()->%d)\n",
+				" (pchan=%s, handle_chan_comb() returns %d)\n",
 				comb, gsm_pchan_name(ts->pchan), rc);
 			talloc_free(tp_merged);
 			/* Send NACK */
@@ -928,17 +982,26 @@ static int oml_rx_set_chan_attr(struct gsm_bts_trx_ts *ts, struct msgb *msg)
 		}
 	}
 
-	/* 9.4.5 ARFCN List */
-
 	/* 9.4.60 TSC */
 	if (TLVP_PRES_LEN(&tp, NM_ATT_TSC, 1)) {
-		ts->tsc = *TLVP_VAL(&tp, NM_ATT_TSC);
+		ts->tsc_oml = ts->tsc = *TLVP_VAL(&tp, NM_ATT_TSC);
+		if (ts->tsc != BTS_TSC(bts) &&
+		    !osmo_bts_has_feature(bts->features, BTS_FEAT_MULTI_TSC)) {
+			LOGPFOH(DOML, LOGL_ERROR, foh, "SET CHAN ATTR: this BTS model does not "
+				"support TSC %u != BSIC-BCC %u\n", ts->tsc, BTS_TSC(bts));
+			talloc_free(tp_merged);
+			return oml_fom_ack_nack(msg, NM_NACK_PARAM_RANGE);
+		}
 	} else {
 		/* If there is no TSC specified, use the BCC */
-		ts->tsc = BSIC2BCC(bts->bsic);
+		ts->tsc_oml = ts->tsc = BTS_TSC(bts);
 	}
-	LOGPFOH(DOML, LOGL_INFO, foh, "%s SET CHAN ATTR (TSC=%u pchan=%s)\n",
-		gsm_abis_mo_name(&ts->mo), ts->tsc, gsm_pchan_name(ts->pchan));
+	LOGPFOH(DOML, LOGL_INFO, foh, "SET CHAN ATTR (TSC=%u pchan=%s",
+		ts->tsc, gsm_pchan_name(ts->pchan));
+	if (ts->hopping.enabled)
+		LOGPC(DOML, LOGL_INFO, " hsn=%u maio=%u chan_num=%u",
+		      ts->hopping.hsn, ts->hopping.maio, ts->hopping.arfcn_num);
+	LOGPC(DOML, LOGL_INFO, ")\n");
 
 	/* call into BTS driver to apply new attributes to hardware */
 	return bts_model_apply_oml(bts, msg, tp_merged, NM_OC_CHANNEL, ts);
@@ -1000,9 +1063,14 @@ static int oml_rx_chg_adm_state(struct gsm_bts *bts, struct msgb *msg)
 		return oml_fom_ack_nack(msg, NM_NACK_OBJINST_UNKN);
 
 	/* Step 2: Do some global dependency/consistency checking */
-	if (mo->nm_state.administrative == adm_state)
+	if (mo->nm_state.administrative == adm_state) {
 		LOGPFOH(DOML, LOGL_NOTICE, foh, "ADM state already was %s\n",
 			get_value_string(abis_nm_adm_state_names, adm_state));
+		return oml_mo_statechg_ack(mo);
+	}
+	LOGPFOH(DOML, LOGL_NOTICE, foh, "ADM STATE %s -> %s\n",
+		get_value_string(abis_nm_adm_state_names, mo->nm_state.administrative),
+		get_value_string(abis_nm_adm_state_names, adm_state));
 
 	/* Step 3: Ask BTS driver to apply the state chg */
 	return bts_model_chg_adm_state(bts, mo, obj, adm_state);
@@ -1114,7 +1182,7 @@ static int down_fom(struct gsm_bts *bts, struct msgb *msg)
  * manufacturer related messages
  */
 
-static int oml_ipa_mo_set_attr_nse(void *obj, struct tlv_parsed *tp)
+static int oml_ipa_mo_set_attr_nse(void *obj, const struct tlv_parsed *tp)
 {
 	struct gsm_bts *bts = container_of(obj, struct gsm_bts, gprs.nse);
 
@@ -1137,7 +1205,7 @@ static int oml_ipa_mo_set_attr_nse(void *obj, struct tlv_parsed *tp)
 	return 0;
 }
 
-static int oml_ipa_mo_set_attr_cell(void *obj, struct tlv_parsed *tp)
+static int oml_ipa_mo_set_attr_cell(void *obj, const struct tlv_parsed *tp)
 {
 	struct gsm_bts *bts = container_of(obj, struct gsm_bts, gprs.cell);
 	struct gprs_rlc_cfg *rlcc = &bts->gprs.cell.rlc_cfg;
@@ -1208,7 +1276,7 @@ static int oml_ipa_mo_set_attr_cell(void *obj, struct tlv_parsed *tp)
 }
 
 static int oml_ipa_mo_set_attr_nsvc(struct gsm_bts_gprs_nsvc *nsvc,
-				    struct tlv_parsed *tp)
+				    const struct tlv_parsed *tp)
 {
 	if (TLVP_PRES_LEN(tp, NM_ATT_IPACC_NSVCI, 2))
 		nsvc->nsvci = ntohs(tlvp_val16_unal(tp, NM_ATT_IPACC_NSVCI));
@@ -1218,23 +1286,67 @@ static int oml_ipa_mo_set_attr_nsvc(struct gsm_bts_gprs_nsvc *nsvc,
 		uint16_t _cur_s;
 		uint32_t _cur_l;
 
+		memset(&nsvc->local, 0, sizeof(nsvc->local));
+		memset(&nsvc->remote, 0, sizeof(nsvc->remote));
+
+		nsvc->local.u.sin.sin_family = AF_INET;
+		nsvc->remote.u.sin.sin_family = AF_INET;
+
 		memcpy(&_cur_s, cur, 2);
-		nsvc->remote_port = ntohs(_cur_s);
+		nsvc->remote.u.sin.sin_port = _cur_s;
 		cur += 2;
 		memcpy(&_cur_l, cur, 4);
-		nsvc->remote_ip = ntohl(_cur_l);
+		nsvc->remote.u.sin.sin_addr.s_addr = _cur_l;
 		cur += 4;
 		memcpy(&_cur_s, cur, 2);
-		nsvc->local_port = ntohs(_cur_s);
+		nsvc->local.u.sin.sin_port = _cur_s;
 	}
 
+	if (TLVP_PRES_LEN(tp, NM_ATT_OSMO_NS_LINK_CFG, 10)) {
+		const uint8_t *cur = TLVP_VAL(tp, NM_ATT_OSMO_NS_LINK_CFG);
+		uint8_t address_family;
+
+		memset(&nsvc->local, 0, sizeof(nsvc->local));
+		memset(&nsvc->remote, 0, sizeof(nsvc->remote));
+
+		address_family = *cur;
+		/* 1byte padding */
+		cur += 2;
+
+		memcpy(&nsvc->local.u.sin.sin_port, cur, 2);
+		cur += 2;
+
+		memcpy(&nsvc->remote.u.sin.sin_port, cur, 2);
+		cur += 2;
+
+		switch (address_family) {
+		case OSMO_NSVC_ADDR_IPV4:
+			/* we already checked for 10 bytes */
+			nsvc->remote.u.sas.ss_family = AF_INET;
+			nsvc->local.u.sas.ss_family = AF_INET;
+			memcpy(&nsvc->remote.u.sin.sin_addr.s_addr, cur, sizeof(in_addr_t));
+			break;
+		case OSMO_NSVC_ADDR_IPV6:
+			if (TLVP_LEN(tp, NM_ATT_OSMO_NS_LINK_CFG) < 22) {
+				return -1;
+			}
+			nsvc->remote.u.sas.ss_family = AF_INET6;
+			nsvc->local.u.sas.ss_family = AF_INET6;
+			memcpy(&nsvc->remote.u.sin6.sin6_addr, cur, sizeof(struct in6_addr));
+			break;
+		default:
+			return -1;
+		}
+	}
+
+
 	osmo_signal_dispatch(SS_GLOBAL, S_NEW_NSVC_ATTR, nsvc);
 
 	return 0;
 }
 
 static int oml_ipa_mo_set_attr(struct gsm_bts *bts, const struct gsm_abis_mo *mo,
-				void *obj, struct tlv_parsed *tp)
+				void *obj, const struct tlv_parsed *tp)
 {
 	int rc;
 
@@ -1286,6 +1398,7 @@ static int oml_ipa_set_attr(struct gsm_bts *bts, struct msgb *msg)
 		/* Success: replace old MO attributes with new */
 		/* merge existing MO attributes with new attributes */
 		tp_merged = osmo_tlvp_copy(mo->nm_attr, bts);
+		talloc_set_name_const(tp_merged, "oml_ipa_attr");
 		osmo_tlvp_merge(tp_merged, &tp);
 		talloc_free(mo->nm_attr);
 		mo->nm_attr = tp_merged;
@@ -1295,36 +1408,46 @@ static int oml_ipa_set_attr(struct gsm_bts *bts, struct msgb *msg)
 }
 
 static int rx_oml_ipa_rsl_connect(struct gsm_bts_trx *trx, struct msgb *msg,
-				  struct tlv_parsed *tp)
+				  const struct tlv_parsed *tp)
 {
 	struct e1inp_sign_link *oml_link = trx->bts->oml_link;
 	uint16_t port = IPA_TCP_PORT_RSL;
-	uint32_t ip = get_signlink_remote_ip(oml_link);
 	const char *trx_name = gsm_trx_name(trx);
 	struct in_addr in;
 	int rc;
 
 	uint8_t stream_id = 0;
 
-	if (TLVP_PRES_LEN(tp, NM_ATT_IPACC_DST_IP, 4)) {
-		ip = ntohl(tlvp_val32_unal(tp, NM_ATT_IPACC_DST_IP));
-	}
-	if (TLVP_PRES_LEN(tp, NM_ATT_IPACC_DST_IP_PORT, 2)) {
-		port = ntohs(tlvp_val16_unal(tp, NM_ATT_IPACC_DST_IP_PORT));
-	}
-	if (TLVP_PRES_LEN(tp, NM_ATT_IPACC_STREAM_ID, 1)) {
-		stream_id = *TLVP_VAL(tp, NM_ATT_IPACC_STREAM_ID);
-	}
+	if (TLVP_PRESENT(tp, NM_ATT_IPACC_DST_IP))
+		in.s_addr = tlvp_val32_unal(tp, NM_ATT_IPACC_DST_IP);
+	else
+		in.s_addr = htonl(get_signlink_remote_ip(oml_link));
 
-	in.s_addr = htonl(ip);
-	LOGP(DOML, LOGL_INFO, "%s: Rx IPA RSL CONNECT IP=%s PORT=%u STREAM=0x%02x\n", 
-		trx_name, inet_ntoa(in), port, stream_id);
+	if (TLVP_PRESENT(tp, NM_ATT_IPACC_DST_IP_PORT))
+		port = ntohs(tlvp_val16_unal(tp, NM_ATT_IPACC_DST_IP_PORT));
+	if (TLVP_PRESENT(tp, NM_ATT_IPACC_STREAM_ID))
+		stream_id = *TLVP_VAL(tp, NM_ATT_IPACC_STREAM_ID);
+
+	LOGP(DOML, LOGL_INFO, "%s: Rx IPA RSL CONNECT IP=%s PORT=%u STREAM=0x%02x\n",
+	     trx_name, inet_ntoa(in), port, stream_id);
 
 	if (trx->bts->variant == BTS_OSMO_OMLDUMMY) {
 		rc = 0;
 		LOGP(DOML, LOGL_NOTICE, "%s: Not connecting RSL in OML-DUMMY!\n", trx_name);
-	} else
+	} else {
+		trx->rsl_tei = stream_id;
 		rc = e1inp_ipa_bts_rsl_connect_n(oml_link->ts->line, inet_ntoa(in), port, trx->nr);
+	}
+
+	/* The ACK/NACK is expected to contain all IEs */
+	if (!TLVP_PRESENT(tp, NM_ATT_IPACC_DST_IP)) /* TV32 */
+		msgb_tv_fixed_put(msg, NM_ATT_IPACC_DST_IP, sizeof(in),
+				  (const uint8_t *) &in);
+	if (!TLVP_PRESENT(tp, NM_ATT_IPACC_DST_IP_PORT)) /* TV16 */
+		msgb_tv16_put(msg, NM_ATT_IPACC_DST_IP_PORT, port);
+	if (!TLVP_PRESENT(tp, NM_ATT_IPACC_STREAM_ID)) /* TV */
+		msgb_tv_put(msg, NM_ATT_IPACC_STREAM_ID, stream_id);
+
 	if (rc < 0) {
 		LOGP(DOML, LOGL_ERROR, "%s: Error in abis_open(RSL): %d\n", trx_name, rc);
 		return oml_fom_ack_nack(msg, NM_NACK_CANT_PERFORM);
@@ -1452,16 +1575,154 @@ int down_oml(struct gsm_bts *bts, struct msgb *msg)
 		ret = -EINVAL;
 	}
 
+	/* msgb was reused, do not free() */
+	if (ret == 1)
+		return 0;
+
 	msgb_free(msg);
 
 	return ret;
 }
 
-int oml_init(struct gsm_abis_mo *mo)
+int oml_init()
 {
 	DEBUGP(DOML, "Initializing OML attribute definitions\n");
 	tlv_def_patch(&abis_nm_att_tlvdef_ipa_local, &abis_nm_att_tlvdef_ipa);
 	tlv_def_patch(&abis_nm_att_tlvdef_ipa_local, &abis_nm_att_tlvdef);
+	tlv_def_patch(&abis_nm_att_tlvdef_ipa_local, &abis_nm_osmo_att_tlvdef);
 
 	return 0;
 }
+
+void gsm_mo_init(struct gsm_abis_mo *mo, struct gsm_bts *bts,
+		 uint8_t obj_class, uint8_t p1, uint8_t p2, uint8_t p3)
+{
+	mo->bts = bts;
+	mo->obj_class = obj_class;
+	mo->obj_inst.bts_nr = p1;
+	mo->obj_inst.trx_nr = p2;
+	mo->obj_inst.ts_nr = p3;
+	mo->nm_state.operational = NM_OPSTATE_DISABLED;
+	mo->nm_state.availability = NM_AVSTATE_POWER_OFF;
+	mo->nm_state.administrative = NM_STATE_LOCKED;
+}
+
+/* obtain the MO structure for a given object instance */
+struct gsm_abis_mo *
+gsm_objclass2mo(struct gsm_bts *bts, uint8_t obj_class,
+	    const struct abis_om_obj_inst *obj_inst)
+{
+	struct gsm_bts_trx *trx;
+	struct gsm_abis_mo *mo = NULL;
+
+	switch (obj_class) {
+	case NM_OC_BTS:
+		mo = &bts->mo;
+		break;
+	case NM_OC_RADIO_CARRIER:
+		if (obj_inst->trx_nr >= bts->num_trx) {
+			return NULL;
+		}
+		trx = gsm_bts_trx_num(bts, obj_inst->trx_nr);
+		mo = &trx->mo;
+		break;
+	case NM_OC_BASEB_TRANSC:
+		if (obj_inst->trx_nr >= bts->num_trx) {
+			return NULL;
+		}
+		trx = gsm_bts_trx_num(bts, obj_inst->trx_nr);
+		mo = &trx->bb_transc.mo;
+		break;
+	case NM_OC_CHANNEL:
+		if (obj_inst->trx_nr >= bts->num_trx) {
+			return NULL;
+		}
+		trx = gsm_bts_trx_num(bts, obj_inst->trx_nr);
+		if (obj_inst->ts_nr >= TRX_NR_TS)
+			return NULL;
+		mo = &trx->ts[obj_inst->ts_nr].mo;
+		break;
+	case NM_OC_SITE_MANAGER:
+		mo = &bts->site_mgr.mo;
+		break;
+	case NM_OC_GPRS_NSE:
+		mo = &bts->gprs.nse.mo;
+		break;
+	case NM_OC_GPRS_CELL:
+		mo = &bts->gprs.cell.mo;
+		break;
+	case NM_OC_GPRS_NSVC:
+		if (obj_inst->trx_nr >= ARRAY_SIZE(bts->gprs.nsvc))
+			return NULL;
+		mo = &bts->gprs.nsvc[obj_inst->trx_nr].mo;
+		break;
+	}
+	return mo;
+}
+
+/* obtain the gsm_nm_state data structure for a given object instance */
+struct gsm_nm_state *
+gsm_objclass2nmstate(struct gsm_bts *bts, uint8_t obj_class,
+		 const struct abis_om_obj_inst *obj_inst)
+{
+	struct gsm_abis_mo *mo;
+
+	mo = gsm_objclass2mo(bts, obj_class, obj_inst);
+	if (!mo)
+		return NULL;
+
+	return &mo->nm_state;
+}
+
+/* obtain the in-memory data structure of a given object instance */
+void *
+gsm_objclass2obj(struct gsm_bts *bts, uint8_t obj_class,
+	     const struct abis_om_obj_inst *obj_inst)
+{
+	struct gsm_bts_trx *trx;
+	void *obj = NULL;
+
+	switch (obj_class) {
+	case NM_OC_BTS:
+		obj = bts;
+		break;
+	case NM_OC_RADIO_CARRIER:
+		if (obj_inst->trx_nr >= bts->num_trx) {
+			return NULL;
+		}
+		trx = gsm_bts_trx_num(bts, obj_inst->trx_nr);
+		obj = trx;
+		break;
+	case NM_OC_BASEB_TRANSC:
+		if (obj_inst->trx_nr >= bts->num_trx) {
+			return NULL;
+		}
+		trx = gsm_bts_trx_num(bts, obj_inst->trx_nr);
+		obj = &trx->bb_transc;
+		break;
+	case NM_OC_CHANNEL:
+		if (obj_inst->trx_nr >= bts->num_trx) {
+			return NULL;
+		}
+		trx = gsm_bts_trx_num(bts, obj_inst->trx_nr);
+		if (obj_inst->ts_nr >= TRX_NR_TS)
+			return NULL;
+		obj = &trx->ts[obj_inst->ts_nr];
+		break;
+	case NM_OC_SITE_MANAGER:
+		obj = &bts->site_mgr;
+		break;
+	case NM_OC_GPRS_NSE:
+		obj = &bts->gprs.nse;
+		break;
+	case NM_OC_GPRS_CELL:
+		obj = &bts->gprs.cell;
+		break;
+	case NM_OC_GPRS_NSVC:
+		if (obj_inst->trx_nr >= ARRAY_SIZE(bts->gprs.nsvc))
+			return NULL;
+		obj = &bts->gprs.nsvc[obj_inst->trx_nr];
+		break;
+	}
+	return obj;
+}
