@@ -15,10 +15,6 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
- *
  */
 
 #include <stdio.h>
@@ -318,9 +314,9 @@ int pcu_tx_info_ind(void)
 		info_ind->flags |= PCU_IF_FLAG_MCS8;
 	if (rlcc->cs_mask & (1 << GPRS_MCS9))
 		info_ind->flags |= PCU_IF_FLAG_MCS9;
-#warning	"isn't dl_tbf_ext wrong?: * 10 and no ntohs"
+	/* FIXME: isn't dl_tbf_ext wrong?: * 10 and no ntohs */
 	info_ind->dl_tbf_ext = rlcc->parameter[T_DL_TBF_EXT];
-#warning	"isn't ul_tbf_ext wrong?: * 10 and no ntohs"
+	/* FIXME: isn't ul_tbf_ext wrong?: * 10 and no ntohs */
 	info_ind->ul_tbf_ext = rlcc->parameter[T_UL_TBF_EXT];
 	info_ind->initial_cs = rlcc->initial_cs;
 	info_ind->initial_mcs = rlcc->initial_mcs;
@@ -888,7 +884,7 @@ static int pcu_rx_act_req(struct gsm_bts *bts,
 		return -EINVAL;
 	}
 	if (act_req->activate)
-		l1sap_chan_act(trx, gsm_lchan2chan_nr(lchan), NULL);
+		l1sap_chan_act(trx, gsm_lchan2chan_nr(lchan));
 	else
 		l1sap_chan_rel(trx, gsm_lchan2chan_nr(lchan));
 
