@@ -1000,14 +1000,14 @@ int find_sched_mframe_idx(enum gsm_phys_chan_config pchan, uint8_t tn)
 }
 
 /* Determine if given frame number contains SACCH (true) or other (false) burst */
-bool trx_sched_is_sacch_fn(struct gsm_bts_trx_ts *ts, uint32_t fn, bool uplink)
+bool trx_sched_is_sacch_fn(const struct gsm_bts_trx_ts *ts, uint32_t fn, bool uplink)
 {
 	int i;
 	const struct trx_sched_multiframe *sched;
 	const struct trx_sched_frame *frame;
 	enum trx_chan_type ch_type;
 
-	i = find_sched_mframe_idx(ts->pchan, ts->nr);
+	i = find_sched_mframe_idx(ts_pchan(ts), ts->nr);
 	if (i < 0)
 		return -EINVAL;
 	sched = &trx_sched_multiframes[i];
