@@ -37,6 +37,7 @@
 #include <osmocom/core/select.h>
 #include <osmocom/core/timer.h>
 #include <osmocom/core/write_queue.h>
+#include <osmocom/core/fsm.h>
 #include <osmocom/gsm/gsm_utils.h>
 #include <osmocom/gsm/lapdm.h>
 
@@ -55,6 +56,7 @@
 #include <osmo-bts/msg_utils.h>
 #include <osmo-bts/dtx_dl_amr_fsm.h>
 #include <osmo-bts/cbch.h>
+#include <osmo-bts/nm_common_fsm.h>
 
 #include <nrw/oc2g/oc2g.h>
 #include <nrw/oc2g/gsml1prim.h>
@@ -604,7 +606,11 @@ static int ph_tch_req(struct gsm_bts_trx *trx, struct msgb *msg,
 		empty_req_from_l1sap(l1p, fl1, u8Tn, u32Fn, sapi, subCh, u8BlockNbr);
 	}
 	/* send message to DSP's queue */
-	osmo_wqueue_enqueue(&fl1->write_q[MQ_L1_WRITE], nmsg);
+	if (osmo_wqueue_enqueue(&fl1->write_q[MQ_L1_WRITE], nmsg) < 0) {
+		LOGPFN(DL1P, LOGL_ERROR, u32Fn, "MQ_L1_WRITE queue full. Dropping msg.\n");
+		msgb_free(nmsg);
+		return -ENOBUFS;
+	}
 	if (dtx_is_first_p1(lchan))
 		dtx_dispatch(lchan, E_FIRST);
 	else
@@ -733,7 +739,7 @@ static enum gsm_phys_chan_config pick_pchan(struct gsm_bts_trx_ts *ts)
 		if (ts->flags & TS_F_PDCH_ACTIVE)
 			return GSM_PCHAN_PDCH;
 		return GSM_PCHAN_TCH_F;
-	case GSM_PCHAN_TCH_F_TCH_H_PDCH:
+	case GSM_PCHAN_OSMO_DYN:
 		return ts->dyn.pchan_is;
 	default:
 		return ts->pchan;
@@ -747,7 +753,7 @@ static uint8_t chan_nr_by_sapi(struct gsm_bts_trx_ts *ts,
 	uint8_t cbits = 0;
 	enum gsm_phys_chan_config pchan = pick_pchan(ts);
 	OSMO_ASSERT(pchan != GSM_PCHAN_TCH_F_PDCH);
-	OSMO_ASSERT(pchan != GSM_PCHAN_TCH_F_TCH_H_PDCH);
+	OSMO_ASSERT(pchan != GSM_PCHAN_OSMO_DYN);
 
 	switch (sapi) {
 	case GsmL1_Sapi_Bcch:
@@ -992,12 +998,9 @@ empty_frame:
 	goto tx;
 }
 
-static void dump_meas_res(int ll, GsmL1_MeasParam_t *m)
-{
-	LOGPC(DL1C, ll, ", Meas: RSSI %-3.2f dBm,  Qual %-3.2f dB,  "
-		"BER %-3.2f,  Timing %d\n", m->fRssi, m->fLinkQuality,
-		m->fBer, m->i16BurstTiming);
-}
+
+#define LOG_FMT_MEAS "Meas: RSSI %-3.2f dBm, Qual %-3.2f dB, BER %-3.2f, Timing %d"
+#define LOG_PARAM_MEAS(meas_param) (meas_param)->fRssi, (meas_param)->fLinkQuality, (meas_param)->fBer, (meas_param)->i16BurstTiming
 
 static int process_meas_res(struct gsm_bts_trx *trx, uint8_t chan_nr,
 				GsmL1_MeasParam_t *m, uint32_t fn)
@@ -1048,10 +1051,10 @@ static int handle_ph_data_ind(struct oc2gl1_hdl *fl1, GsmL1_PhDataInd_t *data_in
 	process_meas_res(trx, chan_nr, &data_ind->measParam, fn);
 
 
-	DEBUGPGT(DL1P, &g_time, "Rx PH-DATA.ind %s (hL2 %08x): %s\n",
+	DEBUGPGT(DL1P, &g_time, "Rx PH-DATA.ind %s (hL2 %08x): %s, " LOG_FMT_MEAS "\n",
 		get_value_string(oc2gbts_l1sapi_names, data_ind->sapi), (uint32_t)data_ind->hLayer2,
-		osmo_hexdump(data_ind->msgUnitParam.u8Buffer, data_ind->msgUnitParam.u8Size));
-	dump_meas_res(LOGL_DEBUG, &data_ind->measParam);
+		osmo_hexdump(data_ind->msgUnitParam.u8Buffer, data_ind->msgUnitParam.u8Size),
+		LOG_PARAM_MEAS(&data_ind->measParam));
 
 	/* check for TCH */
 	if (data_ind->sapi == GsmL1_Sapi_TchF
@@ -1084,11 +1087,10 @@ static int handle_ph_data_ind(struct oc2gl1_hdl *fl1, GsmL1_PhDataInd_t *data_in
 	l1sap->u.data.chan_nr = chan_nr;
 	l1sap->u.data.fn = fn;
 	l1sap->u.data.rssi = rssi;
-	if (!pcu_direct) {
-		l1sap->u.data.ber10k = data_ind->measParam.fBer * 10000;
-		l1sap->u.data.ta_offs_256bits = data_ind->measParam.i16BurstTiming*64;
-		l1sap->u.data.lqual_cb = data_ind->measParam.fLinkQuality * 10;
-	}
+	l1sap->u.data.ber10k = data_ind->measParam.fBer * 10000;
+	l1sap->u.data.ta_offs_256bits = data_ind->measParam.i16BurstTiming*64;
+	l1sap->u.data.lqual_cb = data_ind->measParam.fLinkQuality * 10;
+
 	return l1sap_up(trx, l1sap);
 }
 
@@ -1102,7 +1104,8 @@ static int handle_ph_ra_ind(struct oc2gl1_hdl *fl1, GsmL1_PhRaInd_t *ra_ind,
 	struct ph_rach_ind_param rach_ind_param;
 
 	set_log_ctx_sapi(ra_ind->sapi);
-	dump_meas_res(LOGL_DEBUG, &ra_ind->measParam);
+	LOGPFN(DL1C, LOGL_DEBUG, ra_ind->u32Fn, "Rx PH-RA.ind, " LOG_FMT_MEAS "\n",
+	       LOG_PARAM_MEAS(&ra_ind->measParam));
 
 	if ((ra_ind->msgUnitParam.u8Size != 1) &&
 		(ra_ind->msgUnitParam.u8Size != 2)) {
@@ -1327,17 +1330,12 @@ static int activate_rf_compl_cb(struct gsm_bts_trx *trx, struct msgb *resp,
 			bts_update_status(BTS_STATUS_RF_ACTIVE, 1);
 
 		/* signal availability */
-		oml_mo_state_chg(&trx->mo, NM_OPSTATE_DISABLED, NM_AVSTATE_OK);
-		oml_mo_tx_sw_act_rep(&trx->mo);
-		oml_mo_state_chg(&trx->bb_transc.mo, -1, NM_AVSTATE_OK);
-		oml_mo_tx_sw_act_rep(&trx->bb_transc.mo);
-
-		for (i = 0; i < ARRAY_SIZE(trx->ts); i++)
-			oml_mo_state_chg(&trx->ts[i].mo, NM_OPSTATE_DISABLED, NM_AVSTATE_DEPENDENCY);
+		osmo_fsm_inst_dispatch(trx->mo.fi, NM_EV_SW_ACT, NULL);
+		osmo_fsm_inst_dispatch(trx->bb_transc.mo.fi, NM_EV_SW_ACT, NULL);
 	} else {
 		bts_update_status(BTS_STATUS_RF_ACTIVE, 0);
-		oml_mo_state_chg(&trx->mo, NM_OPSTATE_DISABLED, NM_AVSTATE_OFF_LINE);
-		oml_mo_state_chg(&trx->bb_transc.mo, NM_OPSTATE_DISABLED, NM_AVSTATE_OFF_LINE);
+		osmo_fsm_inst_dispatch(trx->mo.fi, NM_EV_DISABLE, NULL);
+		osmo_fsm_inst_dispatch(trx->bb_transc.mo.fi, NM_EV_DISABLE, NULL);
 	}
 
 	msgb_free(resp);
@@ -1661,7 +1659,7 @@ int l1if_close(struct oc2gl1_hdl *fl1h)
 
 /* TODO(oramadan) MERGE */
 #ifdef MERGE_ME
-static void dsp_alive_compl_cb(struct gsm_bts_trx *trx, struct msgb *resp, void *data)
+static int dsp_alive_compl_cb(struct gsm_bts_trx *trx, struct msgb *resp, void *data)
 {
 	Oc2g_Prim_t *sysp = msgb_sysprim(resp);
 	Oc2g_IsAliveCnf_t *sac = &sysp->u.isAliveCnf;
@@ -1672,9 +1670,10 @@ static void dsp_alive_compl_cb(struct gsm_bts_trx *trx, struct msgb *resp, void
 			get_value_string(oc2gbts_sysprim_names, sysp->id), sac->status, trx->nr);
 
 	msgb_free(resp);
+	return 0;
 }
 
-static int dsp_alive_timer_cb(void *data)
+static void dsp_alive_timer_cb(void *data)
 {
 	struct oc2gl1_hdl *fl1h = data;
 	struct gsm_bts_trx *trx = fl1h->phy_inst->trx;
@@ -1709,7 +1708,7 @@ static int dsp_alive_timer_cb(void *data)
 				/* allocate new list of sent alarms */
 				alarm_sent = talloc_zero(fl1h, struct oml_alarm_list);
 				if (!alarm_sent)
-					return -EIO;
+					return;
 
 				alarm_sent->alarm_signal = S_NM_OML_BTS_DSP_ALIVE_ALARM;
 				/* add alarm to sent list */
@@ -1725,14 +1724,14 @@ static int dsp_alive_timer_cb(void *data)
 	rc = l1if_req_compl(fl1h, msg, dsp_alive_compl_cb, NULL);
 	if (rc < 0) {
 		LOGP(DL1C, LOGL_FATAL, "Failed to send %s primitive\n", get_value_string(oc2gbts_sysprim_names, sys_prim->id));
-		return -EIO;
+		return;
 	}
 
 	/* restart timer */
 	fl1h->hw_alive.dsp_alive_cnt = 0;
 	osmo_timer_schedule(&fl1h->hw_alive.dsp_alive_timer, fl1h->hw_alive.dsp_alive_period, 0);
 
-	return 0;
+	return;
 }
 #endif
 
@@ -1796,8 +1795,7 @@ int bts_model_phy_link_open(struct phy_link *plink)
 	}
 
 	/ * initialize DSP heart beat alive timer * /
-	fl1h->hw_alive.dsp_alive_timer.cb = dsp_alive_timer_cb;
-	fl1h->hw_alive.dsp_alive_timer.data = fl1h;
+	osmo_timer_setup(&fl1h->hw_alive.dsp_alive_timer, dsp_alive_timer_cb, fl1h);
 	fl1h->hw_alive.dsp_alive_cnt = 0;
 	fl1h->hw_alive.dsp_alive_period = pinst->u.oc2g.dsp_alive_period;
 	osmo_timer_schedule(&fl1h->hw_alive.dsp_alive_timer, fl1h->hw_alive.dsp_alive_period, 0); */
