@@ -93,7 +93,7 @@ static const enum GsmL1_LogChComb_t pchan_to_logChComb[_GSM_PCHAN_MAX] = {
 	[GSM_PCHAN_PDCH]		= GsmL1_LogChComb_XIII,
 	[GSM_PCHAN_UNKNOWN]		= GsmL1_LogChComb_0,
 	/*
-	 * GSM_PCHAN_TCH_F_PDCH and GSM_PCHAN_TCH_F_TCH_H_PDCH should not be
+	 * GSM_PCHAN_TCH_F_PDCH and GSM_PCHAN_OSMO_DYN should not be
 	 * part of this, only "real" pchan values will be looked up here.
 	 * See the callers of ts_connect_as().
 	 */
@@ -447,7 +447,7 @@ static int trx_init(struct gsm_bts_trx *trx)
 	dev_par->freqBand = oc2g_band;
 	dev_par->u16Arfcn = trx->arfcn;
 	dev_par->u16BcchArfcn = trx->bts->c0->arfcn;
-	dev_par->u8NbTsc = trx->bts->bsic & 7;
+	dev_par->u8NbTsc = BTS_TSC(trx->bts);
 
 	if (!trx_ms_pwr_ctrl_is_osmo(trx)) {
 		/* Target is in the middle between lower and upper RxLev thresholds */
@@ -540,7 +540,7 @@ static int ts_connect_as(struct gsm_bts_trx_ts *ts,
 	GsmL1_MphConnectReq_t *cr;
 
 	if (pchan == GSM_PCHAN_TCH_F_PDCH
-	    || pchan == GSM_PCHAN_TCH_F_TCH_H_PDCH) {
+	    || pchan == GSM_PCHAN_OSMO_DYN) {
 		LOGP(DL1C, LOGL_ERROR,
 		     "%s Requested TS connect as %s,"
 		     " expected a specific pchan instead\n",
@@ -560,7 +560,7 @@ static int ts_opstart(struct gsm_bts_trx_ts *ts)
 {
 	enum gsm_phys_chan_config pchan = ts->pchan;
 	switch (pchan) {
-	case GSM_PCHAN_TCH_F_TCH_H_PDCH:
+	case GSM_PCHAN_OSMO_DYN:
 		ts->dyn.pchan_is = ts->dyn.pchan_want = GSM_PCHAN_NONE;
 		/* First connect as NONE, until first RSL CHAN ACT. */
 		pchan = GSM_PCHAN_NONE;
@@ -595,7 +595,7 @@ GsmL1_SubCh_t lchan_to_GsmL1_SubCh_t(const struct gsm_lchan *lchan)
 {
 	enum gsm_phys_chan_config pchan = lchan->ts->pchan;
 
-	if (pchan == GSM_PCHAN_TCH_F_TCH_H_PDCH)
+	if (pchan == GSM_PCHAN_OSMO_DYN)
 		pchan = lchan->ts->dyn.pchan_want;
 
 	switch (pchan) {
@@ -614,7 +614,7 @@ GsmL1_SubCh_t lchan_to_GsmL1_SubCh_t(const struct gsm_lchan *lchan)
 	case GSM_PCHAN_PDCH:
 	case GSM_PCHAN_UNKNOWN:
 	default:
-	/* case GSM_PCHAN_TCH_F_TCH_H_PDCH: is caught above */
+	/* case GSM_PCHAN_OSMO_DYN: is caught above */
 		return GsmL1_SubCh_NA;
 	}
 
@@ -1212,10 +1212,6 @@ int lchan_activate(struct gsm_lchan *lchan)
 
 		enqueue_sapi_act_cmd(lchan, sapi, dir);
 	}
-
-#warning "FIXME: Should this be in sapi_activate_cb?"
-	lchan_init_lapdm(lchan);
-
 	return 0;
 }
 
@@ -1836,21 +1832,6 @@ int bts_model_check_oml(struct gsm_bts *bts, uint8_t msg_type,
 			void *obj)
 {
 	/* FIXME: more checks if the attributes are valid */
-
-	switch (msg_type) {
-	case NM_MT_SET_CHAN_ATTR:
-		/* our L1 only supports one global TSC for all channels
-		 * one one TRX, so we need to make sure not to activate
-		 * channels with a different TSC!! */
-		if (TLVP_PRESENT(new_attr, NM_ATT_TSC) &&
-		    TLVP_LEN(new_attr, NM_ATT_TSC) >= 1 &&
-		    *TLVP_VAL(new_attr, NM_ATT_TSC) != (bts->bsic & 7)) {
-			LOGP(DOML, LOGL_ERROR, "Channel TSC %u != BSIC-TSC %u\n",
-				*TLVP_VAL(new_attr, NM_ATT_TSC), bts->bsic & 7);
-			return -NM_NACK_PARAM_RANGE;
-		}
-		break;
-	}
 	return 0;
 }
 
@@ -1858,11 +1839,31 @@ int bts_model_check_oml(struct gsm_bts *bts, uint8_t msg_type,
 int bts_model_apply_oml(struct gsm_bts *bts, struct msgb *msg,
 			struct tlv_parsed *new_attr, int kind, void *obj)
 {
-	if (kind == NM_OC_RADIO_CARRIER) {
-		struct gsm_bts_trx *trx = obj;
-		struct oc2gl1_hdl *fl1h = trx_oc2gl1_hdl(trx);
+	struct abis_om_fom_hdr *foh = msgb_l3(msg);
+	struct gsm_abis_mo *mo = gsm_objclass2mo(bts, foh->obj_class, &foh->obj_inst);
+	struct nm_fsm_ev_setattr_data ev_data = {
+		.msg = msg,
+		.cause = 0,
+	};
+	int rc;
+	struct gsm_bts_trx *trx;
+	struct oc2gl1_hdl *fl1h;
+	uint8_t cell_size;
+
+	/* TODO: NM Object without FSM: */
+	switch (foh->obj_class) {
+	case NM_OC_GPRS_NSE:
+	case NM_OC_GPRS_CELL:
+	case NM_OC_GPRS_NSVC:
+		return oml_fom_ack_nack(ev_data.msg, ev_data.cause);
+	}
+
+	switch (foh->msg_type) {
+	case NM_MT_SET_RADIO_ATTR:
+		trx = obj;
+		fl1h = trx_oc2gl1_hdl(trx);
 		/* convert max TA to max cell size in qbits */
-		uint8_t cell_size = bts->max_ta << 2;
+		cell_size = bts->max_ta << 2;
 
 		/* We do not need to check for L1 handle
 		 * because the max cell size parameter can receive before MphInit */
@@ -1891,11 +1892,14 @@ int bts_model_apply_oml(struct gsm_bts *bts, struct msgb *msg,
 				l1if_set_txpower_c0_idle_pwr_red(fl1h, fl1h->phy_inst->u.oc2g.tx_c0_idle_pwr_red);
 			}
 		}
-
+		break;
 	}
 
-	/* FIXME: we actually need to send a ACK or NACK for the OML message */
-	return oml_fom_ack_nack(msg, 0);
+	rc = osmo_fsm_inst_dispatch(mo->fi,
+				    ev_data.cause == 0 ? NM_EV_SETATTR_ACK : NM_EV_SETATTR_NACK,
+				    &ev_data);
+	/* msgb ownsership is transferred to FSM if it received ev: */
+	return rc == 0 ? 1 : 0;
 }
 
 /* callback from OML */
@@ -1913,10 +1917,10 @@ int bts_model_opstart(struct gsm_bts *bts, struct gsm_abis_mo *mo,
 		break;
 	case NM_OC_BTS:
 		rc = osmo_fsm_inst_dispatch(bts->mo.fi, NM_EV_OPSTART_ACK, NULL);
-		oml_mo_state_chg(&bts->mo, -1, NM_AVSTATE_OK);
-		oml_mo_state_chg(&bts->gprs.nse.mo, -1, NM_AVSTATE_OK);
-		oml_mo_state_chg(&bts->gprs.cell.mo, -1, NM_AVSTATE_OK);
-		oml_mo_state_chg(&bts->gprs.nsvc[0].mo, -1, NM_AVSTATE_OK);
+		oml_mo_state_chg(&bts->mo, -1, NM_AVSTATE_OK, -1);
+		oml_mo_state_chg(&bts->gprs.nse.mo, -1, NM_AVSTATE_OK, -1);
+		oml_mo_state_chg(&bts->gprs.cell.mo, -1, NM_AVSTATE_OK, -1);
+		oml_mo_state_chg(&bts->gprs.nsvc[0].mo, -1, NM_AVSTATE_OK, -1);
 		break;
 	case NM_OC_RADIO_CARRIER:
 		trx = (struct gsm_bts_trx *) obj;
@@ -1933,7 +1937,7 @@ int bts_model_opstart(struct gsm_bts *bts, struct gsm_abis_mo *mo,
 	case NM_OC_GPRS_NSE:
 	case NM_OC_GPRS_CELL:
 	case NM_OC_GPRS_NSVC:
-		oml_mo_state_chg(mo, NM_OPSTATE_ENABLED, -1);
+		oml_mo_state_chg(mo, NM_OPSTATE_ENABLED, -1, -1);
 		rc = oml_mo_opstart_ack(mo);
 		break;
 	default:
