@@ -36,6 +36,7 @@
 
 #include <osmocom/core/talloc.h>
 #include <osmocom/core/socket.h>
+#include <osmocom/core/fsm.h>
 
 #include <osmo-bts/gsm_data.h>
 #include <osmo-bts/bts_model.h>
@@ -43,6 +44,8 @@
 #include <osmo-bts/logging.h>
 #include <osmo-bts/l1sap.h>
 #include <osmo-bts/handover.h>
+#include <osmo-bts/bts.h>
+#include <osmo-bts/nm_common_fsm.h>
 
 #include "l1_if.h"
 #include "l1_oml.h"
@@ -307,22 +310,13 @@ int l1if_req_compl(struct octphy_hdl *fl1h, struct msgb *msg,
 /* For OctPHY, this only about sending state changes to BSC */
 int l1if_activate_rf(struct gsm_bts_trx *trx, int on)
 {
-	int i;
 	if (on) {
 		/* signal availability */
-		oml_mo_state_chg(&trx->mo, NM_OPSTATE_DISABLED, NM_AVSTATE_OK);
-		oml_mo_tx_sw_act_rep(&trx->mo);
-		oml_mo_state_chg(&trx->bb_transc.mo, -1, NM_AVSTATE_OK);
-		oml_mo_tx_sw_act_rep(&trx->bb_transc.mo);
-
-		for (i = 0; i < ARRAY_SIZE(trx->ts); i++)
-			oml_mo_state_chg(&trx->ts[i].mo, NM_OPSTATE_DISABLED,
-					 NM_AVSTATE_DEPENDENCY);
+		osmo_fsm_inst_dispatch(trx->mo.fi, NM_EV_SW_ACT, NULL);
+		osmo_fsm_inst_dispatch(trx->bb_transc.mo.fi, NM_EV_SW_ACT, NULL);
 	} else {
-		oml_mo_state_chg(&trx->mo, NM_OPSTATE_DISABLED,
-				 NM_AVSTATE_OFF_LINE);
-		oml_mo_state_chg(&trx->bb_transc.mo, NM_OPSTATE_DISABLED,
-				 NM_AVSTATE_OFF_LINE);
+		osmo_fsm_inst_dispatch(trx->mo.fi, NM_EV_DISABLE, NULL);
+		osmo_fsm_inst_dispatch(trx->bb_transc.mo.fi, NM_EV_DISABLE, NULL);
 	}
 
 	return 0;
@@ -335,7 +329,7 @@ static enum gsm_phys_chan_config pick_pchan(struct gsm_bts_trx_ts *ts)
 		if (ts->flags & TS_F_PDCH_ACTIVE)
 			return GSM_PCHAN_PDCH;
 		return GSM_PCHAN_TCH_F;
-	case GSM_PCHAN_TCH_F_TCH_H_PDCH:
+	case GSM_PCHAN_OSMO_DYN:
 		return ts->dyn.pchan_is;
 	default:
 		return ts->pchan;
@@ -350,7 +344,7 @@ static uint8_t chan_nr_by_sapi(struct gsm_bts_trx_ts *ts,
 	enum gsm_phys_chan_config pchan = pick_pchan(ts);
 
 	OSMO_ASSERT(pchan != GSM_PCHAN_TCH_F_PDCH);
-	OSMO_ASSERT(pchan != GSM_PCHAN_TCH_F_TCH_H_PDCH);
+	OSMO_ASSERT(pchan != GSM_PCHAN_OSMO_DYN);
 
 	switch (sapi) {
 	case cOCTVC1_GSM_SAPI_ENUM_BCCH:
@@ -779,15 +773,14 @@ int bts_model_init(struct gsm_bts *bts)
 	/* FIXME: what is the nominal transmit power of the PHY/board? */
 	bts->c0->nominal_power = 15;
 
-	gsm_bts_set_feature(bts, BTS_FEAT_GPRS);
-	gsm_bts_set_feature(bts, BTS_FEAT_OML_ALERTS);
+	/* order alphabetically */
 #if defined(cOCTVC1_GSM_LOGICAL_CHANNEL_COMBINATION_ENUM_FCCH_SCH_BCCH_CCCH_SDCCH4_CBCH_SACCHC4) && defined(cOCTVC1_GSM_LOGICAL_CHANNEL_COMBINATION_ENUM_SDCCH8_CBCH_SACCHC8)
-	gsm_bts_set_feature(bts, BTS_FEAT_CBCH);
+	osmo_bts_set_feature(bts->features, BTS_FEAT_CBCH);
 #endif
-	gsm_bts_set_feature(bts, BTS_FEAT_SPEECH_F_V1);
-	gsm_bts_set_feature(bts, BTS_FEAT_SPEECH_H_V1);
-
-	bts_model_vty_init(bts);
+	osmo_bts_set_feature(bts->features, BTS_FEAT_GPRS);
+	osmo_bts_set_feature(bts->features, BTS_FEAT_OML_ALERTS);
+	osmo_bts_set_feature(bts->features, BTS_FEAT_SPEECH_F_V1);
+	osmo_bts_set_feature(bts->features, BTS_FEAT_SPEECH_H_V1);
 
 	return 0;
 }
@@ -906,14 +899,9 @@ static void process_meas_res(struct gsm_bts_trx *trx, uint8_t chan_nr,
 	l1sap_up(trx, &l1sap);
 }
 
-static void dump_meas_res(int ll, tOCTVC1_GSM_MEASUREMENT_INFO * m)
-{
-	LOGP(DMEAS, ll,
-	     "Meas: RSSI %d dBm, Burst Timing %d Quarter of bits :%d, "
-	     "BER Error Count %d , BER Toatal Bit count %d in last decoded frame\n",
-	     m->sRSSIDbm, m->sBurstTiming, m->sBurstTiming4x, m->usBERCnt,
-	     m->usBERTotalBitCnt);
-}
+
+#define LOG_FMT_MEAS "Meas: RSSI %d dBm, Burst Timing %d Quarter of bits: %d, BER Error Count %d, BER Toatal Bit count %d in last decoded frame"
+#define LOG_PARAM_MEAS(meas_param) (meas_param)->sRSSIDbm, (meas_param)->sBurstTiming, (meas_param)->sBurstTiming4x, (meas_param)->usBERCnt, (meas_param)->usBERTotalBitCnt
 
 static int handle_mph_time_ind(struct octphy_hdl *fl1, uint8_t trx_id, uint32_t fn)
 {
@@ -1218,7 +1206,8 @@ static int handle_ph_rach_ind(struct octphy_hdl *fl1,
 
 	set_log_ctx_sapi(ra_ind->LchId.bySAPI);
 
-	dump_meas_res(LOGL_DEBUG, &ra_ind->MeasurementInfo);
+	LOGPFN(DL1C, LOGL_DEBUG, ra_ind->ulFrameNumber, "Rx PH-RA.ind, " LOG_FMT_MEAS "\n",
+	       LOG_PARAM_MEAS(&ra_ind->MeasurementInfo));
 
 	if (ra_ind->ulMsgLength != 1) {
 		LOGPFN(DL1C, LOGL_ERROR, ra_ind->ulFrameNumber,
@@ -1314,7 +1303,11 @@ static int retransmit_wlc_upto(struct octphy_hdl *fl1h, uint32_t trans_id)
 			wlc->num_retrans++;
 			msg = msgb_copy(wlc->cmd_msg, "PHY CMD Retrans");
 			msg_set_retrans_flag(msg);
-			osmo_wqueue_enqueue(&fl1h->phy_wq, msg);
+			if (osmo_wqueue_enqueue(&fl1h->phy_wq, msg) < 0) {
+				LOGP(DL1C, LOGL_ERROR, "Queue full on wlc retransmit\n");
+				msgb_free(msg);
+				return 0;
+			}
 			osmo_timer_schedule(&wlc->timer, CMD_TIMEOUT, 0);
 			count++;
 			LOGP(DL1C, LOGL_INFO, "Re-transmitting %s "
@@ -1829,10 +1822,7 @@ struct octphy_hdl *l1if_open(struct phy_link *plink)
 	osmo_wqueue_init(&fl1h->phy_wq, 10);
 	fl1h->phy_wq.write_cb = octphy_write_cb;
 	fl1h->phy_wq.read_cb = octphy_read_cb;
-	fl1h->phy_wq.bfd.fd = sfd;
-	fl1h->phy_wq.bfd.when = BSC_FD_READ;
-	fl1h->phy_wq.bfd.cb = osmo_wqueue_bfd_cb;
-	fl1h->phy_wq.bfd.data = fl1h;
+	osmo_fd_setup(&fl1h->phy_wq.bfd, sfd, OSMO_FD_READ, osmo_wqueue_bfd_cb, fl1h, 0);
 	rc = osmo_fd_register(&fl1h->phy_wq.bfd);
 	if (rc < 0) {
 		close(sfd);
