@@ -43,7 +43,40 @@
 #include <osmo-bts/msg_utils.h>
 
 #include <sched_utils.h>
-#include <loops.h>
+#include <amr_loop.h>
+
+/* 3GPP TS 45.009, table 3.2.1.3-{2,4}: AMR on Uplink TCH/H.
+ *
+ * +---+---+---+---+---+---+
+ * | a | b | c | d | e | f |  Burst 'a' received first
+ * +---+---+---+---+---+---+
+ *  ^^^^^^^^^^^^^^^^^^^^^^^   FACCH frame  (bursts 'a' .. 'f')
+ *  ^^^^^^^^^^^^^^^           Speech frame (bursts 'a' .. 'd')
+ *
+ * TDMA frame number of burst 'f' is always used as the table index. */
+static const uint8_t sched_tchh_ul_amr_cmi_map[26] = {
+	[10] = 1, /* TCH/H(0): a=0  / d=6  / f=10 */
+	[19] = 1, /* TCH/H(0): a=8  / d=15 / f=19 */
+	[2]  = 1, /* TCH/H(0): a=17 / d=23 / f=2 */
+
+	[11] = 1, /* TCH/H(1): a=1  / d=7  / f=11 */
+	[20] = 1, /* TCH/H(1): a=9  / d=16 / f=20 */
+	[3]  = 1, /* TCH/H(1): a=18 / d=24 / f=3 */
+};
+
+/* 3GPP TS 45.002, table 1 in clause 7: Mapping tables.
+ * TDMA frame number of burst 'f' is always used as the table index. */
+static const uint8_t sched_tchh_ul_facch_map[26] = {
+	[10] = 1, /* FACCH/H(0): B0(0,2,4,6,8,10) */
+	[11] = 1, /* FACCH/H(1): B0(1,3,5,7,9,11) */
+	[19] = 1, /* FACCH/H(0): B1(8,10,13,15,17,19) */
+	[20] = 1, /* FACCH/H(1): B1(9,11,14,16,18,20) */
+	[2]  = 1, /* FACCH/H(0): B2(17,19,21,23,0,2) */
+	[3]  = 1, /* FACCH/H(1): B2(18,20,22,24,1,3) */
+};
+
+/* TDMA frame number of burst 'a' is used as the table index. */
+extern const uint8_t sched_tchh_dl_facch_map[26];
 
 /*! \brief a single TCH/H burst was received by the PHY, process it */
 int rx_tchh_fn(struct l1sched_ts *l1ts, const struct trx_ul_burst_ind *bi)
@@ -55,23 +88,17 @@ int rx_tchh_fn(struct l1sched_ts *l1ts, const struct trx_ul_burst_ind *bi)
 	uint8_t rsl_cmode = chan_state->rsl_cmode;
 	uint8_t tch_mode = chan_state->tch_mode;
 	uint8_t tch_data[128]; /* just to be safe */
-	int rc, amr = 0;
+	int rc = 0; /* initialize to make gcc happy */
+	int amr = 0;
 	int n_errors = 0;
 	int n_bits_total = 0;
 	bool bfi_flag = false;
-	/* Note on FN-10: If we are at FN 10, we decoded an even aligned
-	 * TCH/FACCH frame, because our burst buffer carries 6 bursts.
-	 * Even FN ending at: 10,11,19,20,2,3
-	 */
-	int fn_is_odd = (((bi->fn + 26 - 10) % 26) >> 2) & 1;
-	enum sched_meas_avg_mode meas_avg_mode = SCHED_MEAS_AVG_M_QUAD;
+	enum sched_meas_avg_mode meas_avg_mode = SCHED_MEAS_AVG_M_S6N4;
 	struct l1sched_meas_set meas_avg;
 	unsigned int fn_begin;
-	unsigned int fn_tch_end;
-	uint16_t ber10k;
+	uint16_t ber10k = 0;
 	uint8_t is_sub = 0;
 	uint8_t ft;
-	bool mask_stolen_tch_block = false;
 	bool fn_is_cmi;
 
 	/* If handover RACH detection is turned on, treat this burst as an Access Burst.
@@ -83,15 +110,17 @@ int rx_tchh_fn(struct l1sched_ts *l1ts, const struct trx_ul_burst_ind *bi)
 
 	/* allocate burst memory, if not already */
 	if (!*bursts_p) {
-		*bursts_p = talloc_zero_size(tall_bts_ctx, 696);
+		*bursts_p = talloc_zero_size(l1ts, 696);
 		if (!*bursts_p)
 			return -ENOMEM;
 	}
 
-	/* clear burst */
+	/* shift the buffer by 2 bursts leftwards */
 	if (bi->bid == 0) {
+		memcpy(*bursts_p, *bursts_p + 232, 232);
+		memcpy(*bursts_p + 232, *bursts_p + 464, 232);
 		memset(*bursts_p + 464, 0, 232);
-		*mask = 0x0;
+		*mask = *mask << 2;
 	}
 
 	/* update mask */
@@ -112,43 +141,42 @@ int rx_tchh_fn(struct l1sched_ts *l1ts, const struct trx_ul_burst_ind *bi)
 	if (bi->bid != 1)
 		return 0;
 
-	/* check for complete set of bursts */
-	if ((*mask & 0x3) != 0x3) {
-		LOGL1SB(DL1P, LOGL_NOTICE, l1ts, bi,
-			"Received incomplete frame (%u/%u)\n",
-			bi->fn % l1ts->mf_period, l1ts->mf_period);
+	/* fill up the burst buffer so that we have 6 bursts in there */
+	if (OSMO_UNLIKELY((*mask & 0x3f) != 0x3f)) {
+		LOGL1SB(DL1P, LOGL_DEBUG, l1ts, bi,
+			"UL burst buffer is not filled up: mask=0x%02x != 0x3f\n",
+			*mask);
+		return 0; /* TODO: send BFI */
 	}
-	*mask = 0x0;
 
 	/* skip decoding of the last 4 bursts of FACCH/H */
 	if (chan_state->ul_ongoing_facch) {
 		chan_state->ul_ongoing_facch = 0;
-		memcpy(*bursts_p, *bursts_p + 232, 232);
-		memcpy(*bursts_p + 232, *bursts_p + 464, 232);
 		/* we have already sent the first BFI when a FACCH/H frame
 		 * was decoded (see below), now send the second one. */
-		ber10k = 0;
-		memset(&meas_avg, 0, sizeof(meas_avg));
-		/* In order to provide an even stream of measurement reports
-		 * we ask the code below to mask the missing TCH/H block
-		 * measurement report with the FACCH measurement results. */
-		mask_stolen_tch_block = true;
+		trx_sched_meas_avg(chan_state, &meas_avg, meas_avg_mode);
+		/* meas_avg.fn now contains TDMA frame number of the first burst */
+		fn_begin = meas_avg.fn;
 		goto bfi;
 	}
 
 	/* decode
 	 * also shift buffer by 4 bursts for interleaving */
-	switch ((rsl_cmode != RSL_CMOD_SPD_SPEECH) ? GSM48_CMODE_SPEECH_V1
-								: tch_mode) {
+	switch (tch_mode) {
+	case GSM48_CMODE_SIGN:
+		meas_avg_mode = SCHED_MEAS_AVG_M_S6N6;
+		/* fall-through */
 	case GSM48_CMODE_SPEECH_V1: /* HR or signalling */
-		/* Note on FN-10: If we are at FN 10, we decoded an even aligned
-		 * TCH/FACCH frame, because our burst buffer carries 6 bursts.
-		 * Even FN ending at: 10,11,19,20,2,3
-		 */
 		rc = gsm0503_tch_hr_decode(tch_data, *bursts_p,
-			fn_is_odd, &n_errors, &n_bits_total);
-		if (rc >= 0) /* DTXu */
-			lchan_set_marker(osmo_hr_check_sid(tch_data, rc), lchan);
+					   !sched_tchh_ul_facch_map[bi->fn % 26],
+					   &n_errors, &n_bits_total);
+		if (rc == (GSM_HR_BYTES + 1)) { /* only for valid *speech* frames */
+			/* gsm0503_tch_hr_decode() prepends a ToC octet (see RFC5993), skip it */
+			bool is_sid = osmo_hr_check_sid(&tch_data[1], GSM_HR_BYTES);
+			if (is_sid) /* Mark SID frames as such: F = 0, FT = 010 */
+				tch_data[0] = (0x02 << 4);
+			lchan_set_marker(is_sid, lchan); /* DTXu */
+		}
 		break;
 	case GSM48_CMODE_SPEECH_AMR: /* AMR */
 		/* the first FN 0,8,17 or 1,9,18 defines that CMI is included
@@ -165,28 +193,23 @@ int rx_tchh_fn(struct l1sched_ts *l1ts, const struct trx_ul_burst_ind *bi)
 			break;
 		}
 
-		/* Calculate the frame number where the block begins */
-		if (bi->fn % 13 < 4)
-			fn_tch_end = GSM_TDMA_FN_SUB(bi->fn, 5);
-		else
-			fn_tch_end = GSM_TDMA_FN_SUB(bi->fn, 4);
-		if (lchan->nr == 0)
-			fn_begin = gsm0502_fn_remap(fn_tch_end, FN_REMAP_TCH_H0);
-		else
-			fn_begin = gsm0502_fn_remap(fn_tch_end, FN_REMAP_TCH_H1);
-		fn_is_cmi = ul_amr_fn_is_cmi(fn_begin);
+		fn_is_cmi = sched_tchh_ul_amr_cmi_map[bi->fn % 26];
 
 		/* See comment in function rx_tchf_fn() */
 		amr = 2;
 		rc = gsm0503_tch_ahs_decode_dtx(tch_data + amr, *bursts_p,
-			fn_is_odd, !fn_is_cmi, chan_state->codec,
-			chan_state->codecs, &chan_state->ul_ft,
-			&chan_state->ul_cmr, &n_errors, &n_bits_total, &chan_state->amr_last_dtx);
+						!sched_tchh_ul_facch_map[bi->fn % 26],
+						!fn_is_cmi, chan_state->codec,
+						chan_state->codecs, &chan_state->ul_ft,
+						&chan_state->ul_cmr, &n_errors, &n_bits_total,
+						&chan_state->amr_last_dtx);
 
 		/* Tag all frames that are not regular AMR voice frames
 		   as SUB-Frames */
 		if (chan_state->amr_last_dtx != AMR_OTHER) {
-			LOGL1SB(DL1P, LOGL_DEBUG, l1ts, bi, "Received AMR SID frame: %s\n",
+			LOGL1SB(DL1P, LOGL_DEBUG, l1ts, bi,
+				"Received AMR DTX frame (rc=%d, BER %d/%d): %s\n",
+				rc, n_errors, n_bits_total,
 				gsm0503_amr_dtx_frame_name(chan_state->amr_last_dtx));
 			is_sub = 1;
 		}
@@ -209,16 +232,13 @@ int rx_tchh_fn(struct l1sched_ts *l1ts, const struct trx_ul_burst_ind *bi)
 		case AHS_SID_UPDATE_CN:
 		case AHS_SID_FIRST_INH:
 		case AHS_SID_UPDATE_INH:
-			meas_avg_mode = SCHED_MEAS_AVG_M6_FIRST_TWO;
+			meas_avg_mode = SCHED_MEAS_AVG_M_S6N2;
 			break;
 		case AHS_ONSET:
-			meas_avg_mode = SCHED_MEAS_AVG_M6_MIDDLE_TWO;
+			meas_avg_mode = SCHED_MEAS_AVG_M_S4N2;
 			break;
 		}
 
-		if (rc)
-			trx_loop_amr_input(chan_state, n_errors, n_bits_total);
-
 		/* only good speech frames get rtp header */
 		if (rc != GSM_MACBLOCK_LEN && rc >= 4) {
 			if (chan_state->amr_last_dtx == AMR_OTHER) {
@@ -239,14 +259,16 @@ int rx_tchh_fn(struct l1sched_ts *l1ts, const struct trx_ul_burst_ind *bi)
 			tch_mode);
 		return -EINVAL;
 	}
-	memcpy(*bursts_p, *bursts_p + 232, 232);
-	memcpy(*bursts_p + 232, *bursts_p + 464, 232);
+
 	ber10k = compute_ber10k(n_bits_total, n_errors);
 
 	/* average measurements of the last N (depends on mode) bursts */
-	if (rc == GSM_MACBLOCK_LEN)
-		meas_avg_mode = SCHED_MEAS_AVG_M_SIX;
 	trx_sched_meas_avg(chan_state, &meas_avg, meas_avg_mode);
+	/* meas_avg.fn now contains TDMA frame number of the first burst */
+	fn_begin = meas_avg.fn;
+
+	if (tch_mode == GSM48_CMODE_SPEECH_AMR)
+		trx_loop_amr_input(chan_state, &meas_avg);
 
 	/* Check if the frame is bad */
 	if (rc < 0) {
@@ -269,28 +291,16 @@ int rx_tchh_fn(struct l1sched_ts *l1ts, const struct trx_ul_burst_ind *bi)
 	/* FACCH */
 	if (rc == GSM_MACBLOCK_LEN) {
 		chan_state->ul_ongoing_facch = 1;
-		uint16_t ber10k = compute_ber10k(n_bits_total, n_errors);
-		if (lchan->nr == 0)
-			fn_begin = gsm0502_fn_remap(bi->fn, FN_REMAP_FACCH_H0);
-		else
-			fn_begin = gsm0502_fn_remap(bi->fn, FN_REMAP_FACCH_H1);
+		/* In order to provide an even stream of measurement reports in *speech*
+		 * mode, here we intentionally invalidate RSSI for FACCH, so that this
+		 * report gets dropped in process_l1sap_meas_data().  The averaged results
+		 * will be sent with the first (see below) and second (see above) BFIs. */
 		_sched_compose_ph_data_ind(l1ts, fn_begin, bi->chan,
 			tch_data + amr, GSM_MACBLOCK_LEN,
-			meas_avg.rssi, meas_avg.toa256,
-			meas_avg.ci_cb, ber10k,
+			tch_mode == GSM48_CMODE_SIGN ? meas_avg.rssi : 0,
+			meas_avg.toa256, meas_avg.ci_cb, ber10k,
 			PRES_INFO_UNKNOWN);
-
-		/* Keep a copy of the measurement results of the last FACCH
-		 * transmission in order to be able to create a replacement
-		 * measurement result for the one missing TCH block
-		 * measurement */
-		memcpy(&chan_state->meas_avg_facch, &meas_avg, sizeof(meas_avg));
-		chan_state->ber10k_facch = ber10k;
-
-		/* Invalidate the current measurement result to prevent the
-		 * code below from handing up the current measurement a second
-		 * time. */
-		memset(&meas_avg, 0, sizeof(meas_avg));
+		ber10k = 0;
 bfi:
 		/* A FACCH/H frame replaces two speech frames, so we need to send two BFIs.
 		 * One is sent here, another will be sent two bursts later (see above). */
@@ -341,58 +351,22 @@ bfi:
 
 compose_l1sap:
 	/* TCH or BFI */
-
-	/* The input to gsm0502_fn_remap() needs to get the frame number we
-	 * got two bursts ago. The reason for this is that the burst shift
-	 * buffer we use for decoding is 6 bursts wide (one SACCH block) but
-	 * TCH/H blocks are only 4 bursts wide. The decoder functions look
-	 * at the beginning of the buffer while we shift into it at the end,
-	 * this means that TCH/H blocks always decode delayed by two frame
-	 * number positions late. To calculatue the ending frame number of
-	 * the TCH/H we need to subtract 4 or 5 frames if there was a SACCH
-	 * in between. (Note: this is TCH/H, 4 frames ==> 2 bursts) */
-	if (bi->fn % 13 < 4)
-		fn_tch_end = GSM_TDMA_FN_SUB(bi->fn, 5);
-	else
-		fn_tch_end = GSM_TDMA_FN_SUB(bi->fn, 4);
-	if (lchan->nr == 0)
-		fn_begin = gsm0502_fn_remap(fn_tch_end, FN_REMAP_TCH_H0);
-	else
-		fn_begin = gsm0502_fn_remap(fn_tch_end, FN_REMAP_TCH_H1);
-
-	/* A FACCH/H transmission takes out two TCH/H voice blocks and the
-	 * related measurement results. The first measurement result is handed
-	 * up directly with the FACCH (see above), the second one needs to be
-	 * compensated by filling the gap with the measurement result we got
-	 * from the FACCH transmission. */
-	if (mask_stolen_tch_block) {
-		memcpy(&meas_avg, &chan_state->meas_avg_facch, sizeof(meas_avg));
-		ber10k = chan_state->ber10k_facch;
-		memset(&chan_state->meas_avg_facch, 0, sizeof(meas_avg));
-		chan_state->ber10k_facch = 0;
-	}
-
 	return _sched_compose_tch_ind(l1ts, fn_begin, bi->chan, tch_data, rc,
-				      /* FIXME: what should we use for BFI here? */
-				      bfi_flag ? bi->toa256 : meas_avg.toa256, ber10k,
-				      bfi_flag ? bi->rssi : meas_avg.rssi,
-				      bfi_flag ? bi->ci_cb : meas_avg.ci_cb,
-				      is_sub);
+				      meas_avg.toa256, ber10k, meas_avg.rssi,
+				      meas_avg.ci_cb, is_sub);
 }
 
 /* common section for generation of TCH bursts (TCH/H and TCH/F).
  * FIXME: this function is over-complicated, refactor / get rid of it. */
-extern void tx_tch_common(struct l1sched_ts *l1ts,
-			  const struct trx_dl_burst_req *br,
-			  struct msgb **_msg_tch, struct msgb **_msg_facch);
+extern struct msgb *tch_dl_dequeue(struct l1sched_ts *l1ts, const struct trx_dl_burst_req *br);
 
 /* obtain a to-be-transmitted TCH/H (Half Traffic Channel) burst */
 int tx_tchh_fn(struct l1sched_ts *l1ts, struct trx_dl_burst_req *br)
 {
-	struct msgb *msg_tch = NULL, *msg_facch = NULL;
 	struct l1sched_chan_state *chan_state = &l1ts->chan_state[br->chan];
 	uint8_t tch_mode = chan_state->tch_mode;
 	ubit_t *burst, **bursts_p = &chan_state->dl_bursts;
+	struct msgb *msg;
 
 	/* send burst, if we already got a frame */
 	if (br->bid > 0) {
@@ -401,23 +375,12 @@ int tx_tchh_fn(struct l1sched_ts *l1ts, struct trx_dl_burst_req *br)
 		goto send_burst;
 	}
 
-	/* get TCH and/or FACCH */
-	tx_tch_common(l1ts, br, &msg_tch, &msg_facch);
-
-	/* check for FACCH alignment */
-	if (msg_facch && ((((br->fn + 4) % 26) >> 2) & 1)) {
-		LOGL1SB(DL1P, LOGL_ERROR, l1ts, br,
-			"Cannot transmit FACCH starting on even frames, please fix RTS!\n");
-		msgb_free(msg_facch);
-		msg_facch = NULL;
-	}
-
 	/* BURST BYPASS */
 
 	/* allocate burst memory, if not already,
 	 * otherwise shift buffer by 2 bursts for interleaving */
 	if (!*bursts_p) {
-		*bursts_p = talloc_zero_size(tall_bts_ctx, 696);
+		*bursts_p = talloc_zero_size(l1ts, 696);
 		if (!*bursts_p)
 			return -ENOMEM;
 	} else {
@@ -430,8 +393,18 @@ int tx_tchh_fn(struct l1sched_ts *l1ts, struct trx_dl_burst_req *br)
 		}
 	}
 
+	/* dequeue a message to be transmitted */
+	msg = tch_dl_dequeue(l1ts, br);
+
+	/* if we're sending 2 middle bursts of FACCH/H */
+	if (chan_state->dl_ongoing_facch) {
+		msgb_free(msg); /* drop 2nd speech frame */
+		chan_state->dl_ongoing_facch = 0;
+		goto send_burst;
+	}
+
 	/* no message at all, send a dummy L2 frame on FACCH */
-	if (!msg_tch && !msg_facch && !chan_state->dl_ongoing_facch) {
+	if (msg == NULL) {
 		static const uint8_t dummy[GSM_MACBLOCK_LEN] = {
 			0x03, 0x03, 0x01, /* TODO: use randomized padding */
 			0x2b, 0x2b, 0x2b, 0x2b, 0x2b, 0x2b, 0x2b, 0x2b, 0x2b, 0x2b,
@@ -439,34 +412,40 @@ int tx_tchh_fn(struct l1sched_ts *l1ts, struct trx_dl_burst_req *br)
 		};
 
 		LOGL1SB(DL1P, LOGL_INFO, l1ts, br, "No TCH or FACCH prim for transmit.\n");
+
+		/* FACCH/H can only be scheduled at specific TDMA offset */
+		if (!sched_tchh_dl_facch_map[br->fn % 26]) {
+			/* FACCH/H is not allowed, send half-filled bursts with even numbered
+			 * bits contaning 232 encoded bits of the previous L2 frame, and 232
+			 * odd numbered bits all set to 0. */
+			goto send_burst;
+		}
+
 		gsm0503_tch_hr_encode(*bursts_p, dummy, sizeof(dummy));
+		chan_state->dl_ongoing_facch = 1;
+		chan_state->dl_facch_bursts = 6;
 		goto send_burst;
 	}
 
-	/* encode bursts (prioritize FACCH) */
-	if (msg_facch) {
-		gsm0503_tch_hr_encode(*bursts_p, msg_facch->l2h, msgb_l2len(msg_facch));
+	/* populate the buffer with bursts */
+	if (msgb_l2len(msg) == GSM_MACBLOCK_LEN) {
+		gsm0503_tch_hr_encode(*bursts_p, msg->l2h, msgb_l2len(msg));
 		chan_state->dl_ongoing_facch = 1; /* first of two TCH frames */
 		chan_state->dl_facch_bursts = 6;
-	} else if (chan_state->dl_ongoing_facch) /* second of two TCH frames */
-		chan_state->dl_ongoing_facch = 0; /* we are done with FACCH */
-	else if (tch_mode == GSM48_CMODE_SPEECH_AMR)
+	} else if (tch_mode == GSM48_CMODE_SPEECH_AMR)
 		/* the first FN 4,13,21 or 5,14,22 defines that CMI is included
 		 * in frame, the first FN 0,8,17 or 1,9,18 defines that CMR is
 		 * included in frame. */
-		gsm0503_tch_ahs_encode(*bursts_p, msg_tch->l2h + 2,
-			msgb_l2len(msg_tch) - 2, !dl_amr_fn_is_cmi(br->fn),
+		gsm0503_tch_ahs_encode(*bursts_p, msg->l2h + 2,
+			msgb_l2len(msg) - 2, !dl_amr_fn_is_cmi(br->fn),
 			chan_state->codec, chan_state->codecs,
 			chan_state->dl_ft,
 			chan_state->dl_cmr);
 	else
-		gsm0503_tch_hr_encode(*bursts_p, msg_tch->l2h, msgb_l2len(msg_tch));
+		gsm0503_tch_hr_encode(*bursts_p, msg->l2h, msgb_l2len(msg));
 
 	/* free message */
-	if (msg_tch)
-		msgb_free(msg_tch);
-	if (msg_facch)
-		msgb_free(msg_facch);
+	msgb_free(msg);
 
 send_burst:
 	/* compose burst */
