@@ -58,6 +58,7 @@ int main(int argc, char **argv)
 {
 	void *tall_bts_ctx;
 	struct e1inp_line *line;
+	struct e1inp_ts *sign_ts;
 	struct gsm_lchan *lchan;
 	struct osmo_phsap_prim nl1sap;
 	struct msgb *msg;
@@ -75,14 +76,14 @@ int main(int argc, char **argv)
 		fprintf(stderr, "Failed to create BTS structure\n");
 		exit(1);
 	}
-	trx = gsm_bts_trx_alloc(bts);
-	if (!trx) {
-		fprintf(stderr, "Failed to TRX structure\n");
+	if (bts_init(bts) < 0) {
+		fprintf(stderr, "unable to init BTS\n");
 		exit(1);
 	}
 
-	if (bts_init(bts) < 0) {
-		fprintf(stderr, "unable to to open bts\n");
+	trx = gsm_bts_trx_alloc(bts);
+	if (!trx) {
+		fprintf(stderr, "Failed to alloc TRX structure\n");
 		exit(1);
 	}
 
@@ -90,9 +91,9 @@ int main(int argc, char **argv)
 
 	line = e1inp_line_create(0, "ipa");
 	OSMO_ASSERT(line);
-
-	e1inp_ts_config_sign(&line->ts[E1INP_SIGN_RSL-1], line);
-	trx->rsl_link = e1inp_sign_link_create(&line->ts[E1INP_SIGN_RSL-1], E1INP_SIGN_RSL, NULL, 0, 0);
+	sign_ts = e1inp_line_ipa_rsl_ts(line, 0);
+	e1inp_ts_config_sign(sign_ts, line);
+	trx->rsl_link = e1inp_sign_link_create(sign_ts, E1INP_SIGN_RSL, NULL, 0, 0);
 	OSMO_ASSERT(trx->rsl_link);
 	trx->rsl_link->trx = trx;
 
@@ -101,12 +102,12 @@ int main(int argc, char **argv)
 	/* create two lchans for handover */
 	lchan = &trx->ts[1].lchan[0];
 	lchan->type = GSM_LCHAN_SDCCH;
-	l1sap_chan_act(lchan->ts->trx, 0x09, NULL);
+	l1sap_chan_act(lchan->ts->trx, 0x09);
 	lchan = &trx->ts[2].lchan[0];
 	lchan->type = GSM_LCHAN_TCH_F;
 	lchan->ho.active = HANDOVER_ENABLED;
 	lchan->ho.ref = 23;
-	l1sap_chan_act(lchan->ts->trx, 0x0a, NULL);
+	l1sap_chan_act(lchan->ts->trx, 0x0a);
 	OSMO_ASSERT(msgb_dequeue(&trx->rsl_link->tx_list));
 	OSMO_ASSERT(msgb_dequeue(&trx->rsl_link->tx_list));
 	OSMO_ASSERT(!msgb_dequeue(&trx->rsl_link->tx_list));
@@ -114,6 +115,8 @@ int main(int argc, char **argv)
 	/* send access burst with wrong ref */
 	memset(&nl1sap, 0, sizeof(nl1sap));
 	osmo_prim_init(&nl1sap.oph, SAP_GSM_PH, PRIM_PH_RACH, PRIM_OP_INDICATION, NULL);
+	/* Report the minimum acceptable value to pass L1SAP checks */
+	nl1sap.u.rach_ind.lqual_cb = bts->min_qual_rach;
 	nl1sap.u.rach_ind.chan_nr = 0x0a;
 	nl1sap.u.rach_ind.ra = 42;
 	l1sap_up(trx, &nl1sap);
@@ -262,17 +265,3 @@ done:
 		msgb_free(msg);
 	return rc;
 }
-
-int bts_model_check_oml(struct gsm_bts *bts, uint8_t msg_type, struct tlv_parsed *old_attr, struct tlv_parsed *new_attr, void *obj) { return 0; }
-int bts_model_apply_oml(struct gsm_bts *bts, struct msgb *msg, struct tlv_parsed *new_attr, int obj_kind, void *obj) { return 0; }
-int bts_model_opstart(struct gsm_bts *bts, struct gsm_abis_mo *mo, void *obj) { return 0; }
-int bts_model_chg_adm_state(struct gsm_bts *bts, struct gsm_abis_mo *mo, void *obj, uint8_t adm_state) { return 0; }
-int bts_model_init(struct gsm_bts *bts) { return 0; }
-int bts_model_trx_deact_rf(struct gsm_bts_trx *trx) { return 0; }
-int bts_model_trx_close(struct gsm_bts_trx *trx) { return 0; }
-void trx_get_hlayer1(void) {}
-int bts_model_adjst_ms_pwr(struct gsm_lchan *lchan) { return 0; }
-int bts_model_ts_disconnect(struct gsm_bts_trx_ts *ts) { return 0; }
-int bts_model_ts_connect(struct gsm_bts_trx_ts *ts, enum gsm_phys_chan_config as_pchan) { return 0; }
-int bts_model_lchan_deactivate(struct gsm_lchan *lchan) { return 0; }
-int bts_model_lchan_deactivate_sacch(struct gsm_lchan *lchan) { return 0; }
