@@ -84,11 +84,12 @@ static void test_meas_compute(const struct meas_testcase *mtc)
 
 	lchan = &trx->ts[mtc->ts].lchan[0];
 	lchan->ts->pchan = mtc->pchan;
+	lchan->tch_mode = GSM48_CMODE_SPEECH_V1;
 	reset_lchan_meas(lchan);
 
 	/* feed uplink measurements into the code */
 	for (i = 0; i < mtc->ulm_count; i++) {
-		lchan_new_ul_meas(lchan, (struct bts_ul_meas *) &mtc->ulm[i], fn);
+		lchan_new_ul_meas(lchan, &mtc->ulm[i], fn);
 		fn += 1;
 	}
 
@@ -328,7 +329,7 @@ void test_lchan_meas_process_measurement(bool no_sacch, bool dropouts)
 
 	ulm.ber10k = 0;
 	ulm.ta_offs_256bits = 256;
-	ulm.c_i = 0;
+	ulm.ci_cb = 0;
 	ulm.is_sub = 0;
 	ulm.inv_rssi = 90;
 
@@ -372,73 +373,27 @@ void test_lchan_meas_process_measurement(bool no_sacch, bool dropouts)
 	}
 }
 
-static bool test_ts45008_83_is_sub_is_sacch(uint32_t fn)
-{
-	if (fn % 104 == 12)
-		return true;
-	if (fn % 104 == 25)
-		return true;
-	if (fn % 104 == 38)
-		return true;
-	if (fn % 104 == 51)
-		return true;
-	if (fn % 104 == 64)
-		return true;
-	if (fn % 104 == 77)
-		return true;
-	if (fn % 104 == 90)
-		return true;
-	if (fn % 104 == 103)
-		return true;
-
-	return false;
-}
-
-static bool test_ts45008_83_is_sub_is_sub(uint32_t fn, uint8_t ss)
+static bool test_ts45008_83_is_sub_is_sub(const struct gsm_lchan *lchan, uint32_t fn)
 {
 	fn = fn % 104;
 
-	if (fn >= 52 && fn <= 59)
-		return true;
-
-	if (ss == 0) {
-		if (fn == 0)
+	switch (lchan->type) {
+	case GSM_LCHAN_TCH_F:
+		/* block {52, 53, 54, 55, 56, 57, 58, 59} */
+		return fn == 52;
+	case GSM_LCHAN_TCH_H:
+		if (fn == 0)	/* H0 block { 0,  2,  4,  6} */
 			return true;
-		if (fn == 2)
+		if (fn == 52)	/* H0 block {52, 54, 56, 58} */
 			return true;
-		if (fn == 4)
+		if (fn == 14)	/* H1 block {14, 16, 18, 20} */
 			return true;
-		if (fn == 6)
+		if (fn == 66)	/* H1 block {66, 68, 70, 72} */
 			return true;
-		if (fn == 52)
-			return true;
-		if (fn == 54)
-			return true;
-		if (fn == 56)
-			return true;
-		if (fn == 58)
-			return true;
-	} else if (ss == 1) {
-		if (fn == 14)
-			return true;
-		if (fn == 16)
-			return true;
-		if (fn == 18)
-			return true;
-		if (fn == 20)
-			return true;
-		if (fn == 66)
-			return true;
-		if (fn == 68)
-			return true;
-		if (fn == 70)
-			return true;
-		if (fn == 72)
-			return true;
-	} else
-		OSMO_ASSERT(false);
-
-	return false;
+		return false;
+	default:
+		return false;
+	}
 }
 
 static void test_ts45008_83_is_sub_single(uint8_t ts, uint8_t ss, bool fr)
@@ -463,29 +418,17 @@ static void test_ts45008_83_is_sub_single(uint8_t ts, uint8_t ss, bool fr)
 		lchan->tch_mode = GSM48_CMODE_SPEECH_V1;
 	}
 
-	printf(" TS=%u ", ts);
-	printf("SS=%u", ss);
+	printf(" TS=%u SS=%u\n", ts, ss);
 
 	/* Walk trough the first 100 intervals and check for unexpected
 	 * results (false positive and false negative) */
 	for (i = 0; i < 104 * 100; i++) {
-		rc = ts45008_83_is_sub(lchan, i, false);
-		if (rc) {
-			if (!test_ts45008_83_is_sub_is_sacch(i)
-			    && !test_ts45008_83_is_sub_is_sub(i, ss)) {
-				printf("==> Unexpected SUB frame at fn=%u", i);
-				OSMO_ASSERT(false);
-			}
-		} else {
-			if (test_ts45008_83_is_sub_is_sacch(i)
-			    && test_ts45008_83_is_sub_is_sub(i, ss)) {
-				printf("==> Unexpected non-SUB frame at fn=%u",
-				       i);
-				OSMO_ASSERT(false);
-			}
+		rc = ts45008_83_is_sub(lchan, i);
+		if (rc != test_ts45008_83_is_sub_is_sub(lchan, i)) {
+			printf("  ==> ts45008_83_is_sub(fn=%u) yields %s, expected %s\n",
+			       i, rc ? "true" : "false", !rc ? "true" : "false");
 		}
 	}
-	printf("\n");
 }
 
 static void test_ts45008_83_is_sub(void)
@@ -529,10 +472,6 @@ int main(int argc, char **argv)
 		fprintf(stderr, "Failed to alloc TRX structure\n");
 		exit(1);
 	}
-	if (bts_trx_init(trx) < 0) {
-		fprintf(stderr, "unable to init TRX\n");
-		exit(1);
-	}
 
 	printf("\n");
 	printf("***********************\n");
@@ -588,88 +527,3 @@ int main(int argc, char **argv)
 
 	return 0;
 }
-
-/* Stubs */
-void bts_model_abis_close(struct gsm_bts *bts)
-{
-}
-
-int bts_model_oml_estab(struct gsm_bts *bts)
-{
-	return 0;
-}
-
-int bts_model_l1sap_down(struct gsm_bts_trx *trx, struct osmo_phsap_prim *l1sap)
-{
-	return 0;
-}
-
-int bts_model_check_oml(struct gsm_bts *bts, uint8_t msg_type, struct tlv_parsed *old_attr, struct tlv_parsed *new_attr,
-			void *obj)
-{
-	return 0;
-}
-
-int bts_model_apply_oml(struct gsm_bts *bts, struct msgb *msg, struct tlv_parsed *new_attr, int obj_kind, void *obj)
-{
-	return 0;
-}
-
-int bts_model_opstart(struct gsm_bts *bts, struct gsm_abis_mo *mo, void *obj)
-{
-	return 0;
-}
-
-int bts_model_chg_adm_state(struct gsm_bts *bts, struct gsm_abis_mo *mo, void *obj, uint8_t adm_state)
-{
-	return 0;
-}
-
-int bts_model_init(struct gsm_bts *bts)
-{
-	return 0;
-}
-
-int bts_model_trx_init(struct gsm_bts_trx *trx)
-{
-	return 0;
-}
-
-int bts_model_trx_deact_rf(struct gsm_bts_trx *trx)
-{
-	return 0;
-}
-
-int bts_model_trx_close(struct gsm_bts_trx *trx)
-{
-	return 0;
-}
-
-void trx_get_hlayer1(void)
-{
-}
-
-int bts_model_adjst_ms_pwr(struct gsm_lchan *lchan)
-{
-	return 0;
-}
-
-int bts_model_ts_disconnect(struct gsm_bts_trx_ts *ts)
-{
-	return 0;
-}
-
-void bts_model_ts_connect(struct gsm_bts_trx_ts *ts, enum gsm_phys_chan_config as_pchan)
-{
-	return;
-}
-
-int bts_model_lchan_deactivate(struct gsm_lchan *lchan)
-{
-	return 0;
-}
-
-int bts_model_lchan_deactivate_sacch(struct gsm_lchan *lchan)
-{
-	return 0;
-}
